@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -27,9 +26,103 @@
 #include "../Application/jucer_Headers.h"
 #include "jucer_Application.h"
 #include "jucer_MainWindow.h"
-#include "../Wizards/jucer_NewProjectWizardClasses.h"
+#include "StartPage/jucer_StartPageComponent.h"
 #include "../Utility/UI/jucer_JucerTreeViewBase.h"
 #include "../ProjectSaving/jucer_ProjectSaver.h"
+#include "UserAccount/jucer_LoginFormComponent.h"
+#include "../Project/UI/jucer_ProjectContentComponent.h"
+
+//==============================================================================
+class BlurOverlayWithComponent  : public Component,
+                                  private ComponentMovementWatcher,
+                                  private AsyncUpdater
+{
+public:
+    BlurOverlayWithComponent (MainWindow& window, std::unique_ptr<Component> comp)
+        : ComponentMovementWatcher (&window),
+          mainWindow (window),
+          componentToShow (std::move (comp))
+    {
+        kernel.createGaussianBlur (1.25f);
+
+        addAndMakeVisible (*componentToShow);
+
+        setAlwaysOnTop (true);
+        setOpaque (true);
+        setVisible (true);
+
+        static_cast<Component&> (mainWindow).addChildComponent (this);
+        componentMovedOrResized (true, true);
+
+        enterModalState();
+    }
+
+    void resized() override
+    {
+        setBounds (mainWindow.getLocalBounds());
+        componentToShow->centreWithSize (componentToShow->getWidth(), componentToShow->getHeight());
+        refreshBackgroundImage();
+    }
+
+    void paint (Graphics& g) override
+    {
+        g.drawImage (componentImage, getLocalBounds().toFloat());
+    }
+
+    void inputAttemptWhenModal() override
+    {
+        mainWindow.hideLoginFormOverlay();
+    }
+
+private:
+    void componentPeerChanged() override                {}
+
+    void componentVisibilityChanged() override          {}
+    using ComponentMovementWatcher::componentVisibilityChanged;
+
+    void componentMovedOrResized (bool, bool) override  { triggerAsyncUpdate(); }
+    using ComponentMovementWatcher::componentMovedOrResized;
+
+    void handleAsyncUpdate() override                   { resized(); }
+
+    void mouseUp (const MouseEvent& event) override
+    {
+        if (event.eventComponent == this)
+            mainWindow.hideLoginFormOverlay();
+    }
+
+    void lookAndFeelChanged() override
+    {
+        refreshBackgroundImage();
+        repaint();
+    }
+
+    void refreshBackgroundImage()
+    {
+        setAlwaysOnTop (false);
+        toBack();
+
+        auto parentBounds = mainWindow.getBounds();
+
+        componentImage = mainWindow.createComponentSnapshot (mainWindow.getLocalBounds())
+                                   .rescaled (roundToInt ((float) parentBounds.getWidth() / 1.75f),
+                                              roundToInt ((float) parentBounds.getHeight() / 1.75f));
+
+        kernel.applyToImage (componentImage, componentImage, getLocalBounds());
+
+        setAlwaysOnTop (true);
+        toFront (true);
+    }
+
+    //==============================================================================
+    MainWindow& mainWindow;
+    std::unique_ptr<Component> componentToShow;
+
+    ImageConvolutionKernel kernel { 3 };
+    Image componentImage;
+
+    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (BlurOverlayWithComponent)
+};
 
 //==============================================================================
 MainWindow::MainWindow()
@@ -40,6 +133,8 @@ MainWindow::MainWindow()
                       false)
 {
     setUsingNativeTitleBar (true);
+    setResizable (true, false);
+    setResizeLimits (600, 500, 32000, 32000);
 
    #if ! JUCE_MAC
     setMenuBar (ProjucerApplication::getApp().getMenuModel());
@@ -47,9 +142,6 @@ MainWindow::MainWindow()
 
     createProjectContentCompIfNeeded();
 
-    setResizable (true, false);
-    centreWithSize (800, 600);
-
     auto& commandManager = ProjucerApplication::getCommandManager();
 
     auto registerAllAppCommands = [&]
@@ -73,9 +165,10 @@ MainWindow::MainWindow()
 
     setWantsKeyboardFocus (false);
     getLookAndFeel().setColour (ColourSelector::backgroundColourId, Colours::transparentBlack);
+
     projectNameValue.addListener (this);
 
-    setResizeLimits (600, 500, 32000, 32000);
+    centreWithSize (800, 600);
 }
 
 MainWindow::~MainWindow()
@@ -85,10 +178,11 @@ MainWindow::~MainWindow()
    #endif
 
     removeKeyListener (ProjucerApplication::getCommandManager().getKeyMappings());
+
+    // save the current size and position to our settings file..
     getGlobalProperties().setValue ("lastMainWindowPos", getWindowStateAsString());
 
     clearContentComponent();
-    currentProject.reset();
 }
 
 void MainWindow::createProjectContentCompIfNeeded()
@@ -100,7 +194,7 @@ void MainWindow::createProjectContentCompIfNeeded()
     }
 }
 
-void MainWindow::setTitleBarIcon()
+void MainWindow::updateTitleBarIcon()
 {
     if (auto* peer = getPeer())
     {
@@ -121,7 +215,7 @@ void MainWindow::makeVisible()
     setVisible (true);
     addToDesktop();
     restoreWindowPosition();
-    setTitleBarIcon();
+    updateTitleBarIcon();
     getContentComponent()->grabKeyboardFocus();
 }
 
@@ -135,54 +229,97 @@ void MainWindow::closeButtonPressed()
     ProjucerApplication::getApp().mainWindowList.closeWindow (this);
 }
 
-bool MainWindow::closeCurrentProject (bool askUserToSave)
+void MainWindow::closeCurrentProject (OpenDocumentManager::SaveIfNeeded askUserToSave, std::function<void (bool)> callback)
 {
     if (currentProject == nullptr)
-        return true;
+    {
+        if (callback != nullptr)
+            callback (true);
+
+        return;
+    }
 
     currentProject->getStoredProperties().setValue (getProjectWindowPosName(), getWindowStateAsString());
 
     if (auto* pcc = getProjectContentComponent())
     {
-        pcc->saveTreeViewState();
         pcc->saveOpenDocumentList();
         pcc->hideEditor();
     }
 
-    if (ProjucerApplication::getApp().openDocumentManager
-         .closeAllDocumentsUsingProject (*currentProject, askUserToSave))
+    ProjucerApplication::getApp().openDocumentManager
+        .closeAllDocumentsUsingProjectAsync (*currentProject,
+                                             askUserToSave,
+                                             [parent = SafePointer<MainWindow> { this }, askUserToSave, callback] (bool closedSuccessfully)
     {
-        if (! askUserToSave || (currentProject->saveIfNeededAndUserAgrees() == FileBasedDocument::savedOk))
-        {
-            setProject (nullptr);
-            return true;
-        }
-    }
+        if (parent == nullptr)
+            return;
 
-    return false;
+        if (! closedSuccessfully)
+        {
+            if (callback != nullptr)
+                callback (false);
+
+            return;
+        }
+
+        auto setProjectAndCallback = [parent, callback]
+        {
+            parent->setProject (nullptr);
+
+            if (callback != nullptr)
+                callback (true);
+        };
+
+        if (askUserToSave == OpenDocumentManager::SaveIfNeeded::no)
+        {
+            setProjectAndCallback();
+            return;
+        }
+
+        parent->currentProject->saveIfNeededAndUserAgreesAsync ([parent, setProjectAndCallback, callback] (FileBasedDocument::SaveResult saveResult)
+        {
+            if (parent == nullptr)
+                return;
+
+            if (saveResult == FileBasedDocument::savedOk)
+                setProjectAndCallback();
+            else if (callback != nullptr)
+                callback (false);
+        });
+    });
 }
 
-void MainWindow::moveProject (File newProjectFileToOpen)
+void MainWindow::moveProject (File newProjectFileToOpen, OpenInIDE openInIDE)
 {
-    auto openInIDE = currentProject->shouldOpenInIDEAfterSaving();
-
-    closeCurrentProject (false);
-    openFile (newProjectFileToOpen);
-
-    if (currentProject != nullptr)
+    closeCurrentProject (OpenDocumentManager::SaveIfNeeded::no,
+                         [parent = SafePointer<MainWindow> { this }, newProjectFileToOpen, openInIDE] (bool)
     {
-        ProjucerApplication::getApp().getCommandManager().invokeDirectly (openInIDE ? CommandIDs::saveAndOpenInIDE
-                                                                                    : CommandIDs::saveProject,
-                                                                          false);
-    }
+        if (parent == nullptr)
+            return;
+
+        parent->openFile (newProjectFileToOpen, [parent, openInIDE] (bool openedSuccessfully)
+        {
+            if (! (openedSuccessfully && parent != nullptr && parent->currentProject != nullptr && openInIDE == OpenInIDE::yes))
+                return;
+
+            // The project component knows how to process the saveAndOpenInIDE command, but the
+            // main application does not. In order to process the command successfully, we need
+            // to ensure that the project content component has focus.
+            auto& manager = ProjucerApplication::getApp().getCommandManager();
+            manager.setFirstCommandTarget (parent->getProjectContentComponent());
+            ProjucerApplication::getApp().getCommandManager().invokeDirectly (CommandIDs::saveAndOpenInIDE, false);
+            manager.setFirstCommandTarget (nullptr);
+        });
+    });
 }
 
 void MainWindow::setProject (std::unique_ptr<Project> newProject)
 {
     if (newProject == nullptr)
     {
-        getProjectContentComponent()->setProject (nullptr);
-        projectNameValue.referTo (Value());
+        if (auto* content = getProjectContentComponent())
+            content->setProject (nullptr);
 
         currentProject.reset();
     }
@@ -192,12 +329,16 @@ void MainWindow::setProject (std::unique_ptr<Project> newProject)
 
         createProjectContentCompIfNeeded();
         getProjectContentComponent()->setProject (currentProject.get());
-        projectNameValue.referTo (currentProject->getProjectValue (Ids::name));
-
-        if (auto* peer = getPeer())
-            peer->setRepresentedFile (currentProject->getFile());
     }
 
+    if (currentProject != nullptr)
+        currentProject->addChangeListener (this);
+
+    changeListenerCallback (currentProject.get());
+
+    projectNameValue.referTo (currentProject != nullptr ? currentProject->getProjectValue (Ids::name) : Value());
+    initialiseProjectWindow();
+
     ProjucerApplication::getCommandManager().commandStatusChanged();
 }
 
@@ -221,166 +362,169 @@ bool MainWindow::canOpenFile (const File& file) const
                   || ProjucerApplication::getApp().openDocumentManager.canOpenFile (file));
 }
 
-bool MainWindow::openFile (const File& file)
+void MainWindow::openFile (const File& file, std::function<void (bool)> callback)
 {
-    createProjectContentCompIfNeeded();
-
     if (file.hasFileExtension (Project::projectFileExtension))
     {
         auto newDoc = std::make_unique<Project> (file);
         auto result = newDoc->loadFrom (file, true);
 
-        if (result.wasOk() && closeCurrentProject (true))
+        if (result.wasOk())
         {
-            setProject (std::move (newDoc));
-            currentProject->setChangedFlag (false);
+            closeCurrentProject (OpenDocumentManager::SaveIfNeeded::yes,
+                                 [parent = SafePointer<MainWindow> { this },
+                                  sharedDoc = std::make_shared<std::unique_ptr<Project>> (std::move (newDoc)),
+                                  callback] (bool saveResult)
+            {
+                if (parent == nullptr)
+                    return;
 
-            getProjectContentComponent()->reloadLastOpenDocuments();
-            currentProject->updateDeprecatedProjectSettingsInteractively();
+                if (saveResult)
+                {
+                    parent->setProject (std::move (*sharedDoc.get()));
+                    parent->currentProject->setChangedFlag (false);
 
-            return true;
+                    parent->createProjectContentCompIfNeeded();
+                    parent->getProjectContentComponent()->reloadLastOpenDocuments();
+
+                    parent->currentProject->updateDeprecatedProjectSettingsInteractively();
+                }
+
+                if (callback != nullptr)
+                    callback (saveResult);
+            });
+
+            return;
         }
-    }
-    else if (file.exists())
-    {
-        return getProjectContentComponent()->showEditorForFile (file, true);
+
+        if (callback != nullptr)
+            callback (false);
+
+        return;
     }
 
-    return false;
+    if (file.exists())
+    {
+        SafePointer<MainWindow> parent { this };
+        auto createCompAndShowEditor = [parent, file, callback]
+        {
+            if (parent != nullptr)
+            {
+                parent->createProjectContentCompIfNeeded();
+
+                if (callback != nullptr)
+                    callback (parent->getProjectContentComponent()->showEditorForFile (file, true));
+            }
+        };
+
+        if (isPIPFile (file))
+        {
+            openPIP (file, [parent, createCompAndShowEditor, callback] (bool openedSuccessfully)
+            {
+                if (parent == nullptr)
+                    return;
+
+                if (openedSuccessfully)
+                {
+                    if (callback != nullptr)
+                        callback (true);
+
+                    return;
+                }
+
+                createCompAndShowEditor();
+            });
+
+            return;
+        }
+
+        createCompAndShowEditor();
+        return;
+    }
+
+    if (callback != nullptr)
+        callback (false);
 }
 
-bool MainWindow::tryToOpenPIP (const File& pipFile)
+void MainWindow::openPIP (const File& pipFile, std::function<void (bool)> callback)
 {
-    PIPGenerator generator (pipFile);
+    auto generator = std::make_shared<PIPGenerator> (pipFile);
 
-    if (! generator.hasValidPIP())
-        return false;
+    if (! generator->hasValidPIP())
+    {
+        if (callback != nullptr)
+            callback (false);
 
-    auto generatorResult = generator.createJucerFile();
+        return;
+    }
+
+    auto generatorResult = generator->createJucerFile();
 
     if (generatorResult != Result::ok())
     {
-        AlertWindow::showMessageBoxAsync (AlertWindow::WarningIcon,
+        AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
                                           "PIP Error.",
                                           generatorResult.getErrorMessage());
 
-        return false;
+        if (callback != nullptr)
+            callback (false);
+
+        return;
     }
 
-
-    if (! generator.createMainCpp())
+    if (! generator->createMainCpp())
     {
-        AlertWindow::showMessageBoxAsync (AlertWindow::WarningIcon,
+        AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
                                           "PIP Error.",
                                           "Failed to create Main.cpp.");
 
-        return false;
+        if (callback != nullptr)
+            callback (false);
+
+        return;
     }
 
-    if (! ProjucerApplication::getApp().mainWindowList.openFile (generator.getJucerFile()))
-        return false;
-
-    openPIP (generator);
-    return true;
-}
-
-static bool isDivider (const String& line)
-{
-    auto afterIndent = line.trim();
-
-    if (afterIndent.startsWith ("//") && afterIndent.length() > 20)
+    openFile (generator->getJucerFile(), [parent = SafePointer<MainWindow> { this }, generator, callback] (bool openedSuccessfully)
     {
-        afterIndent = afterIndent.substring (2);
+        if (parent == nullptr)
+            return;
 
-        if (afterIndent.containsOnly ("=")
-            || afterIndent.containsOnly ("/")
-            || afterIndent.containsOnly ("-"))
+        if (! openedSuccessfully)
         {
-            return true;
+            AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
+                                              "PIP Error.",
+                                              "Failed to open .jucer file.");
+
+            if (callback != nullptr)
+                callback (false);
+
+            return;
         }
-    }
 
-    return false;
+        parent->setupTemporaryPIPProject (*generator);
+
+        if (callback != nullptr)
+            callback (true);
+    });
 }
 
-static bool isEndOfCommentBlock (const String& line)
+void MainWindow::setupTemporaryPIPProject (PIPGenerator& generator)
 {
-    if (line.contains ("*/"))
-        return true;
+    jassert (currentProject != nullptr);
 
-    return false;
-}
+    currentProject->setTemporaryDirectory (generator.getOutputDirectory());
 
-static int getIndexOfCommentBlockStart (const StringArray& lines, int blockEndIndex)
-{
-    for (int i = blockEndIndex; i >= 0; --i)
+    if (auto* pcc = getProjectContentComponent())
     {
-        if (lines[i].contains ("/*"))
-            return i;
-    }
+        auto fileToDisplay = generator.getPIPFile();
 
-    return  0;
-}
-
-static int findBestLineToScrollTo (StringArray lines, StringRef className)
-{
-    for (auto line : lines)
-    {
-        if (line.contains ("struct " + className) || line.contains ("class " + className))
+        if (fileToDisplay != File())
         {
-            auto index = lines.indexOf (line);
+            pcc->showEditorForFile (fileToDisplay, true);
 
-            if (isDivider (lines[index - 1]))
-                return index - 1;
-
-            if (isEndOfCommentBlock (lines[index - 1]))
-            {
-                auto blockStartIndex = getIndexOfCommentBlockStart (lines, index - 1);
-
-                if (blockStartIndex > 0 && isDivider (lines [blockStartIndex - 1]))
-                    return blockStartIndex - 1;
-
-                return blockStartIndex;
-            }
-
-            return lines.indexOf (line);
-        }
-    }
-
-    return 0;
-}
-
-void MainWindow::openPIP (PIPGenerator& generator)
-{
-    if (auto* window = ProjucerApplication::getApp().mainWindowList.getMainWindowForFile (generator.getJucerFile()))
-    {
-        if (auto* project = window->getProject())
-        {
-            project->setTemporaryDirectory (generator.getOutputDirectory());
-
-            ProjectSaver liveBuildSaver (*project, project->getFile());
-            liveBuildSaver.saveContentNeededForLiveBuild();
-
-            if (auto* pcc = window->getProjectContentComponent())
-            {
-                pcc->invokeDirectly (CommandIDs::toggleBuildEnabled, true);
-                pcc->invokeDirectly (CommandIDs::buildNow, true);
-                pcc->invokeDirectly (CommandIDs::toggleContinuousBuild, true);
-
-                auto fileToDisplay = generator.getPIPFile();
-
-                if (fileToDisplay != File())
-                {
-                    pcc->showEditorForFile (fileToDisplay, true);
-
-                    if (auto* sourceCodeEditor = dynamic_cast <SourceCodeEditor*> (pcc->getEditorComponent()))
-                    {
-                        sourceCodeEditor->editor->scrollToLine (findBestLineToScrollTo (StringArray::fromLines (fileToDisplay.loadFileAsString()),
-                                                                                        generator.getMainClassName()));
-                    }
-                }
-
-            }
+            if (auto* sourceCodeEditor = dynamic_cast <SourceCodeEditor*> (pcc->getEditorComponent()))
+                sourceCodeEditor->editor->scrollToLine (findBestLineToScrollToForClass (StringArray::fromLines (fileToDisplay.loadFileAsString()),
+                                                                                        generator.getMainClassName(), currentProject->getProjectType().isAudioPlugin()));
         }
     }
 }
@@ -394,18 +538,32 @@ bool MainWindow::isInterestedInFileDrag (const StringArray& filenames)
     return false;
 }
 
+static void filesDroppedRecursive (Component::SafePointer<MainWindow> parent, StringArray filenames)
+{
+    if (filenames.isEmpty())
+        return;
+
+    auto f = filenames[0];
+    filenames.remove (0);
+
+    if (! parent->canOpenFile (f))
+    {
+        filesDroppedRecursive (parent, filenames);
+        return;
+    }
+
+    parent->openFile (f, [parent, filenames] (bool openedSuccessfully)
+    {
+        if (parent == nullptr || ! openedSuccessfully)
+            return;
+
+        filesDroppedRecursive (parent, filenames);
+    });
+}
+
 void MainWindow::filesDropped (const StringArray& filenames, int /*mouseX*/, int /*mouseY*/)
 {
-    for (auto& filename : filenames)
-    {
-        const File f (filename);
-
-        if (tryToOpenPIP (f))
-            continue;
-
-        if (! isPIPFile (f) && (canOpenFile (f) && openFile (f)))
-            break;
-    }
+    filesDroppedRecursive (this, filenames);
 }
 
 bool MainWindow::shouldDropFilesWhenDraggedExternally (const DragAndDropTarget::SourceDetails& sourceDetails,
@@ -448,64 +606,43 @@ void MainWindow::activeWindowStatusChanged()
         pcc->updateMissingFileStatuses();
 
     ProjucerApplication::getApp().openDocumentManager.reloadModifiedFiles();
+}
 
-    if (auto* p = getProject())
-    {
-        if (p->hasProjectBeenModified())
-        {
-            Component::SafePointer<Component> safePointer (this);
-
-            MessageManager::callAsync ([=] ()
-            {
-                if (safePointer == nullptr)
-                    return; // bail out if the window has been deleted
-
-                auto result = AlertWindow::showOkCancelBox (AlertWindow::QuestionIcon,
-                                                            TRANS ("The .jucer file has been modified since the last save."),
-                                                            TRANS ("Do you want to keep the current project or re-load from disk?"),
-                                                            TRANS ("Keep"),
-                                                            TRANS ("Re-load from disk"));
-
-                if (safePointer == nullptr)
-                    return;
-
-                if (result == 0)
-                {
-                    if (auto* project = getProject())
-                    {
-                        auto oldTemporaryDirectory = project->getTemporaryDirectory();
-
-                        auto projectFile = project->getFile();
-                        setProject (nullptr);
-                        openFile (projectFile);
-
-                        if (oldTemporaryDirectory != File())
-                            if (auto* newProject = getProject())
-                                newProject->setTemporaryDirectory (oldTemporaryDirectory);
-                    }
-                }
-                else
-                {
-                    ProjucerApplication::getApp().getCommandManager().invokeDirectly (CommandIDs::saveProject, true);
-                }
-            });
-        }
-    }
+void MainWindow::initialiseProjectWindow()
+{
+    setResizable (true, false);
+    updateTitleBarIcon();
 }
 
 void MainWindow::showStartPage()
 {
     jassert (currentProject == nullptr);
 
-    setContentOwned (createNewProjectWizardComponent(), true);
+    setContentOwned (new StartPageComponent ([this] (std::unique_ptr<Project>&& newProject) { setProject (std::move (newProject)); },
+                                             [this] (const File& exampleFile) { openFile (exampleFile, nullptr); }),
+                     true);
 
-    centreWithSize (900, 630);
-    setVisible (true);
+    setResizable (false, false);
+    setName ("New Project");
     addToDesktop();
+    centreWithSize (getContentComponent()->getWidth(), getContentComponent()->getHeight());
 
+    setVisible (true);
     getContentComponent()->grabKeyboardFocus();
 }
 
+void MainWindow::showLoginFormOverlay()
+{
+    blurOverlayComponent = std::make_unique<BlurOverlayWithComponent> (*this, std::make_unique<LoginFormComponent> (*this));
+    loginFormOpen = true;
+}
+
+void MainWindow::hideLoginFormOverlay()
+{
+    blurOverlayComponent.reset();
+    loginFormOpen = false;
+}
+
 //==============================================================================
 ApplicationCommandTarget* MainWindow::getNextCommandTarget()
 {
@@ -573,12 +710,21 @@ bool MainWindow::perform (const InvocationInfo& info)
     return true;
 }
 
-void MainWindow::valueChanged (Value&)
+void MainWindow::valueChanged (Value& value)
 {
-    if (currentProject != nullptr)
-        setName (currentProject->getProjectNameString() + " - Projucer");
-    else
-        setName ("Projucer");
+    if (value == projectNameValue)
+        setName (currentProject != nullptr ? currentProject->getProjectNameString() + " - Projucer"
+                                           : "Projucer");
+}
+
+void MainWindow::changeListenerCallback (ChangeBroadcaster* source)
+{
+    auto* project = getProject();
+
+    if (source == project)
+        if (auto* peer = getPeer())
+            peer->setHasChangedSinceSaved (project != nullptr ? project->hasChangedSinceSaved()
+                                                              : false);
 }
 
 //==============================================================================
@@ -591,19 +737,38 @@ void MainWindowList::forceCloseAllWindows()
     windows.clear();
 }
 
-bool MainWindowList::askAllWindowsToClose()
+static void askAllWindowsToCloseRecursive (WeakReference<MainWindowList> parent, std::function<void (bool)> callback)
 {
-    saveCurrentlyOpenProjectList();
-
-    while (windows.size() > 0)
+    if (parent->windows.size() == 0)
     {
-        if (! windows[0]->closeCurrentProject (true))
-            return false;
+        if (callback != nullptr)
+            callback (true);
 
-        windows.remove (0);
+        return;
     }
 
-    return true;
+    parent->windows[0]->closeCurrentProject (OpenDocumentManager::SaveIfNeeded::yes, [parent, callback] (bool closedSuccessfully)
+    {
+        if (parent == nullptr)
+            return;
+
+        if (! closedSuccessfully)
+        {
+            if (callback != nullptr)
+                callback (false);
+
+            return;
+        }
+
+        parent->windows.remove (0);
+        askAllWindowsToCloseRecursive (parent, std::move (callback));
+    });
+}
+
+void MainWindowList::askAllWindowsToClose (std::function<void (bool)> callback)
+{
+    saveCurrentlyOpenProjectList();
+    askAllWindowsToCloseRecursive (this, std::move (callback));
 }
 
 void MainWindowList::createWindowIfNoneAreOpen()
@@ -624,11 +789,18 @@ void MainWindowList::closeWindow (MainWindow* w)
     else
    #endif
     {
-        if (w->closeCurrentProject (true))
-        {
-            windows.removeObject (w);
-            saveCurrentlyOpenProjectList();
-        }
+        w->closeCurrentProject (OpenDocumentManager::SaveIfNeeded::yes,
+                                [parent = WeakReference<MainWindowList> { this }, w] (bool closedSuccessfully)
+                                {
+                                    if (parent == nullptr)
+                                        return;
+
+                                    if (closedSuccessfully)
+                                    {
+                                        parent->windows.removeObject (w);
+                                        parent->saveCurrentlyOpenProjectList();
+                                    }
+                                });
     }
 }
 
@@ -664,46 +836,70 @@ void MainWindowList::openDocument (OpenDocumentManager::Document* doc, bool grab
     getFrontmostWindow()->getProjectContentComponent()->showDocument (doc, grabFocus);
 }
 
-bool MainWindowList::openFile (const File& file, bool openInBackground)
+void MainWindowList::openFile (const File& file, std::function<void (bool)> callback, bool openInBackground)
 {
+    if (! file.exists())
+    {
+        if (callback != nullptr)
+            callback (false);
+
+        return;
+    }
+
     for (auto* w : windows)
     {
         if (w->getProject() != nullptr && w->getProject()->getFile() == file)
         {
             w->toFront (true);
-            return true;
+
+            if (callback != nullptr)
+                callback (true);
+
+            return;
         }
     }
 
-    if (file.hasFileExtension (Project::projectFileExtension))
+    WeakReference<MainWindowList> parent { this };
+
+    if (file.hasFileExtension (Project::projectFileExtension)
+        || isPIPFile (file))
     {
         WeakReference<Component> previousFrontWindow (getFrontmostWindow());
 
         auto* w = getOrCreateEmptyWindow();
         jassert (w != nullptr);
 
-        if (w->openFile (file))
+        w->openFile (file, [parent, previousFrontWindow, w, openInBackground, callback] (bool openedSuccessfully)
         {
-            w->makeVisible();
-            checkWindowBounds (*w);
+            if (parent == nullptr)
+                return;
 
-            if (openInBackground && previousFrontWindow != nullptr)
-                previousFrontWindow->toFront (true);
+            if (openedSuccessfully)
+            {
+                w->makeVisible();
+                w->setResizable (true, false);
+                parent->checkWindowBounds (*w);
 
-            return true;
-        }
+                if (openInBackground && previousFrontWindow != nullptr)
+                    previousFrontWindow->toFront (true);
+            }
+            else
+            {
+                parent->closeWindow (w);
+            }
 
-        closeWindow (w);
-        return false;
+            if (callback != nullptr)
+                callback (openedSuccessfully);
+        });
+
+        return;
     }
 
-    if (getFrontmostWindow()->tryToOpenPIP (file))
-        return true;
-
-    if (! isPIPFile (file) && file.exists())
-        return getFrontmostWindow()->openFile (file);
-
-    return false;
+    getFrontmostWindow()->openFile (file, [parent, callback] (bool openedSuccessfully)
+    {
+        if (parent != nullptr && callback != nullptr)
+            callback (openedSuccessfully);
+    });
 }
 
 MainWindow* MainWindowList::createNewMainWindow()
@@ -774,6 +970,15 @@ MainWindow* MainWindowList::getMainWindowForFile (const File& file)
     return nullptr;
 }
 
+MainWindow* MainWindowList::getMainWindowWithLoginFormOpen()
+{
+    for (auto* window : windows)
+        if (window->isShowingLoginForm())
+            return window;
+
+    return nullptr;
+}
+
 void MainWindowList::checkWindowBounds (MainWindow& windowToCheck)
 {
     auto avoidSuperimposedWindows = [&]
@@ -804,7 +1009,7 @@ void MainWindowList::checkWindowBounds (MainWindow& windowToCheck)
     auto ensureWindowIsFullyOnscreen = [&]
     {
         auto windowBounds = windowToCheck.getScreenBounds();
-        auto screenLimits = Desktop::getInstance().getDisplays().findDisplayForRect (windowBounds).userArea;
+        auto screenLimits = Desktop::getInstance().getDisplays().getDisplayForRect (windowBounds)->userArea;
 
         if (auto* peer = windowToCheck.getPeer())
             peer->getFrameSize().subtractFrom (screenLimits);
@@ -844,7 +1049,7 @@ void MainWindowList::reopenLastProjects()
 
     for (auto& p : getAppSettings().getLastProjects())
         if (p.existsAsFile())
-            openFile (p, true);
+            openFile (p, nullptr, true);
 }
 
 void MainWindowList::sendLookAndFeelChange()
