@@ -90,15 +90,15 @@ public:
     }
 
     //==============================================================================
-    ValueWithDefault androidJavaLibs, androidAdditionalJavaFolders, androidAdditionalResourceFolders, androidProjectRepositories,
-                     androidRepositories, androidDependencies, androidCustomAppBuildGradleContent, androidScreenOrientation,
-                     androidCustomActivityClass, androidCustomApplicationClass, androidManifestCustomXmlElements,
-                     androidGradleSettingsContent, androidVersionCode, androidMinimumSDK, androidTargetSDK, androidTheme,
-                     androidExtraAssetsFolder, androidOboeRepositoryPath, androidInternetNeeded, androidMicNeeded, androidCameraNeeded,
-                     androidBluetoothNeeded, androidExternalReadPermission, androidExternalWritePermission,
-                     androidInAppBillingPermission, androidVibratePermission, androidOtherPermissions, androidPushNotifications,
-                     androidEnableRemoteNotifications, androidRemoteNotificationsConfigFile, androidEnableContentSharing, androidKeyStore,
-                     androidKeyStorePass, androidKeyAlias, androidKeyAliasPass, gradleVersion, gradleToolchain, androidPluginVersion;
+    ValueTreePropertyWithDefault androidJavaLibs, androidAdditionalJavaFolders, androidAdditionalResourceFolders, androidProjectRepositories,
+                                 androidRepositories, androidDependencies, androidCustomAppBuildGradleContent, androidScreenOrientation,
+                                 androidCustomActivityClass, androidCustomApplicationClass, androidManifestCustomXmlElements,
+                                 androidGradleSettingsContent, androidVersionCode, androidMinimumSDK, androidTargetSDK, androidTheme,
+                                 androidExtraAssetsFolder, androidOboeRepositoryPath, androidInternetNeeded, androidMicNeeded, androidCameraNeeded,
+                                 androidBluetoothNeeded, androidExternalReadPermission, androidExternalWritePermission,
+                                 androidInAppBillingPermission, androidVibratePermission, androidOtherPermissions, androidPushNotifications,
+                                 androidEnableRemoteNotifications, androidRemoteNotificationsConfigFile, androidEnableContentSharing, androidKeyStore,
+                                 androidKeyStorePass, androidKeyAlias, androidKeyAliasPass, gradleVersion, gradleToolchain, androidPluginVersion;
 
     //==============================================================================
     AndroidProjectExporter (Project& p, const ValueTree& t)
@@ -141,7 +141,7 @@ public:
           gradleVersion                        (settings, Ids::gradleVersion,                        getUndoManager(), "7.0.2"),
           gradleToolchain                      (settings, Ids::gradleToolchain,                      getUndoManager(), "clang"),
           androidPluginVersion                 (settings, Ids::androidPluginVersion,                 getUndoManager(), "7.0.0"),
-          androidExecutable                    (getAppSettings().getStoredPath (Ids::androidStudioExePath, TargetOS::getThisOS()).get().toString())
+          AndroidExecutable                    (getAppSettings().getStoredPath (Ids::androidStudioExePath, TargetOS::getThisOS()).get().toString())
     {
         name = getDisplayName();
         targetLocationValue.setDefault (getDefaultBuildsRootFolder() + getTargetFolderName());
@@ -165,25 +165,22 @@ public:
     //==============================================================================
     bool canLaunchProject() override
     {
-        return androidExecutable.exists();
+        return AndroidExecutable.exists();
     }
 
     bool launchProject() override
     {
-        if (! androidExecutable.exists())
+        if (! AndroidExecutable.exists())
         {
             jassertfalse;
             return false;
         }
 
+        auto targetFolder = getTargetFolder();
+
         // we have to surround the path with extra quotes, otherwise Android Studio
         // will choke if there are any space characters in the path.
-        return androidExecutable.startAsProcess (getIDEProjectFile().getFullPathName().quoted());
-    }
-
-    File getIDEProjectFile() const override
-    {
-        return getTargetFolder();
+        return AndroidExecutable.startAsProcess ("\"" + targetFolder.getFullPathName() + "\"");
     }
 
     //==============================================================================
@@ -335,9 +332,9 @@ protected:
             return "${ANDROID_ABI}";
         }
 
-        ValueWithDefault androidArchitectures, androidBuildConfigRemoteNotifsConfigFile,
-                         androidAdditionalXmlValueResources, androidAdditionalDrawableResources,
-                         androidAdditionalRawValueResources, androidCustomStringXmlElements;
+        ValueTreePropertyWithDefault androidArchitectures, androidBuildConfigRemoteNotifsConfigFile,
+                                     androidAdditionalXmlValueResources, androidAdditionalDrawableResources,
+                                     androidAdditionalRawValueResources, androidCustomStringXmlElements;
     };
 
     BuildConfiguration::Ptr createBuildConfig (const ValueTree& v) const override
@@ -1877,7 +1874,7 @@ private:
     }
 
     //==============================================================================
-    const File androidExecutable;
+    const File AndroidExecutable;
 
     JUCE_DECLARE_NON_COPYABLE (AndroidProjectExporter)
 };
