@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -23,21 +23,41 @@
 namespace juce
 {
 
+template <typename Setup>
+static auto getSetupInfo (Setup& s, bool isInput)
+{
+    struct SetupInfo
+    {
+        // double brackets so that we get the expression type, i.e. a (possibly const) reference
+        decltype ((s.inputDeviceName)) name;
+        decltype ((s.inputChannels)) channels;
+        decltype ((s.useDefaultInputChannels)) useDefault;
+    };
+
+    return isInput ? SetupInfo { s.inputDeviceName,  s.inputChannels,  s.useDefaultInputChannels }
+                   : SetupInfo { s.outputDeviceName, s.outputChannels, s.useDefaultOutputChannels };
+}
+
+static auto tie (const AudioDeviceManager::AudioDeviceSetup& s)
+{
+    return std::tie (s.outputDeviceName,
+                     s.inputDeviceName,
+                     s.sampleRate,
+                     s.bufferSize,
+                     s.inputChannels,
+                     s.useDefaultInputChannels,
+                     s.outputChannels,
+                     s.useDefaultOutputChannels);
+}
+
 bool AudioDeviceManager::AudioDeviceSetup::operator== (const AudioDeviceManager::AudioDeviceSetup& other) const
 {
-    return outputDeviceName == other.outputDeviceName
-            && inputDeviceName == other.inputDeviceName
-            && sampleRate == other.sampleRate
-            && bufferSize == other.bufferSize
-            && inputChannels == other.inputChannels
-            && useDefaultInputChannels == other.useDefaultInputChannels
-            && outputChannels == other.outputChannels
-            && useDefaultOutputChannels == other.useDefaultOutputChannels;
+    return tie (*this) == tie (other);
 }
 
 bool AudioDeviceManager::AudioDeviceSetup::operator!= (const AudioDeviceManager::AudioDeviceSetup& other) const
 {
-    return ! operator== (other);
+    return tie (*this) != tie (other);
 }
 
 //==============================================================================
@@ -109,33 +129,77 @@ void AudioDeviceManager::createDeviceTypesIfNeeded()
 
         types.clear (false);
 
-        if (auto* first = availableDeviceTypes.getFirst())
-            currentDeviceType = first->getTypeName();
+        for (auto* type : availableDeviceTypes)
+            type->scanForDevices();
+
+        pickCurrentDeviceTypeWithDevices();
     }
 }
 
+void AudioDeviceManager::pickCurrentDeviceTypeWithDevices()
+{
+    const auto deviceTypeHasDevices = [] (const AudioIODeviceType* ptr)
+    {
+        return ! ptr->getDeviceNames (true) .isEmpty()
+            || ! ptr->getDeviceNames (false).isEmpty();
+    };
+
+    if (auto* type = findType (currentDeviceType))
+        if (deviceTypeHasDevices (type))
+            return;
+
+    const auto iter = std::find_if (availableDeviceTypes.begin(),
+                                    availableDeviceTypes.end(),
+                                    deviceTypeHasDevices);
+
+    if (iter != availableDeviceTypes.end())
+        currentDeviceType = (*iter)->getTypeName();
+}
+
 const OwnedArray<AudioIODeviceType>& AudioDeviceManager::getAvailableDeviceTypes()
 {
     scanDevicesIfNeeded();
     return availableDeviceTypes;
 }
 
+void AudioDeviceManager::updateCurrentSetup()
+{
+    if (currentAudioDevice != nullptr)
+    {
+        currentSetup.sampleRate     = currentAudioDevice->getCurrentSampleRate();
+        currentSetup.bufferSize     = currentAudioDevice->getCurrentBufferSizeSamples();
+        currentSetup.inputChannels  = currentAudioDevice->getActiveInputChannels();
+        currentSetup.outputChannels = currentAudioDevice->getActiveOutputChannels();
+    }
+}
+
 void AudioDeviceManager::audioDeviceListChanged()
 {
     if (currentAudioDevice != nullptr)
     {
-        auto isCurrentDeviceStillAvailable = [&]
+        auto currentDeviceStillAvailable = [&]
         {
-            for (auto* dt : availableDeviceTypes)
-                if (currentAudioDevice->getTypeName() == dt->getTypeName())
-                    for (auto& dn : dt->getDeviceNames())
-                        if (currentAudioDevice->getName() == dn)
+            auto currentTypeName = currentAudioDevice->getTypeName();
+            auto currentDeviceName = currentAudioDevice->getName();
+
+            for (auto* deviceType : availableDeviceTypes)
+            {
+                if (currentTypeName == deviceType->getTypeName())
+                {
+                    for (auto& deviceName : deviceType->getDeviceNames (true))
+                        if (currentDeviceName == deviceName)
                             return true;
 
-            return false;
-        };
+                    for (auto& deviceName : deviceType->getDeviceNames (false))
+                        if (currentDeviceName == deviceName)
+                            return true;
+                }
+            }
 
-        if (! isCurrentDeviceStillAvailable())
+            return false;
+        }();
+
+        if (! currentDeviceStillAvailable)
         {
             closeAudioDevice();
 
@@ -145,13 +209,7 @@ void AudioDeviceManager::audioDeviceListChanged()
                 initialiseDefault (preferredDeviceName, &currentSetup);
         }
 
-        if (currentAudioDevice != nullptr)
-        {
-            currentSetup.sampleRate     = currentAudioDevice->getCurrentSampleRate();
-            currentSetup.bufferSize     = currentAudioDevice->getCurrentBufferSizeSamples();
-            currentSetup.inputChannels  = currentAudioDevice->getActiveInputChannels();
-            currentSetup.outputChannels = currentAudioDevice->getActiveOutputChannels();
-        }
+        updateCurrentSetup();
     }
 
     sendChangeMessage();
@@ -166,8 +224,9 @@ static void addIfNotNull (OwnedArray<AudioIODeviceType>& list, AudioIODeviceType
 
 void AudioDeviceManager::createAudioDeviceTypes (OwnedArray<AudioIODeviceType>& list)
 {
-    addIfNotNull (list, AudioIODeviceType::createAudioIODeviceType_WASAPI (false));
-    addIfNotNull (list, AudioIODeviceType::createAudioIODeviceType_WASAPI (true));
+    addIfNotNull (list, AudioIODeviceType::createAudioIODeviceType_WASAPI (WASAPIDeviceMode::shared));
+    addIfNotNull (list, AudioIODeviceType::createAudioIODeviceType_WASAPI (WASAPIDeviceMode::exclusive));
+    addIfNotNull (list, AudioIODeviceType::createAudioIODeviceType_WASAPI (WASAPIDeviceMode::sharedLowLatency));
     addIfNotNull (list, AudioIODeviceType::createAudioIODeviceType_DirectSound());
     addIfNotNull (list, AudioIODeviceType::createAudioIODeviceType_ASIO());
     addIfNotNull (list, AudioIODeviceType::createAudioIODeviceType_CoreAudio());
@@ -227,6 +286,7 @@ String AudioDeviceManager::initialise (const int numInputChannelsNeeded,
                                        const AudioDeviceSetup* preferredSetupOptions)
 {
     scanDevicesIfNeeded();
+    pickCurrentDeviceTypeWithDevices();
 
     numInputChansNeeded = numInputChannelsNeeded;
     numOutputChansNeeded = numOutputChannelsNeeded;
@@ -250,26 +310,67 @@ String AudioDeviceManager::initialiseDefault (const String& preferredDefaultDevi
     }
     else if (preferredDefaultDeviceName.isNotEmpty())
     {
-        for (auto* type : availableDeviceTypes)
+        const auto nameMatches = [&preferredDefaultDeviceName] (const String& name)
         {
-            for (auto& out : type->getDeviceNames (false))
+            return name.matchesWildcard (preferredDefaultDeviceName, true);
+        };
+
+        struct WildcardMatch
+        {
+            String value;
+            bool successful;
+        };
+
+        const auto getWildcardMatch = [&nameMatches] (const StringArray& names)
+        {
+            const auto iter = std::find_if (names.begin(), names.end(), nameMatches);
+            return WildcardMatch { iter != names.end() ? *iter : String(), iter != names.end() };
+        };
+
+        struct WildcardMatches
+        {
+            WildcardMatch input, output;
+        };
+
+        const auto getMatchesForType = [&getWildcardMatch] (const AudioIODeviceType* type)
+        {
+            return WildcardMatches { getWildcardMatch (type->getDeviceNames (true)),
+                                     getWildcardMatch (type->getDeviceNames (false)) };
+        };
+
+        struct SearchResult
+        {
+            String type, input, output;
+        };
+
+        const auto result = [&]
+        {
+            // First, look for a device type with an input and output which matches the preferred name
+            for (auto* type : availableDeviceTypes)
             {
-                if (out.matchesWildcard (preferredDefaultDeviceName, true))
-                {
-                    setup.outputDeviceName = out;
-                    break;
-                }
+                const auto matches = getMatchesForType (type);
+
+                if (matches.input.successful && matches.output.successful)
+                    return SearchResult { type->getTypeName(), matches.input.value, matches.output.value };
             }
 
-            for (auto& in : type->getDeviceNames (true))
+            // No device type has matching ins and outs, so fall back to a device where either the
+            // input or output match
+            for (auto* type : availableDeviceTypes)
             {
-                if (in.matchesWildcard (preferredDefaultDeviceName, true))
-                {
-                    setup.inputDeviceName = in;
-                    break;
-                }
+                const auto matches = getMatchesForType (type);
+
+                if (matches.input.successful || matches.output.successful)
+                    return SearchResult { type->getTypeName(), matches.input.value, matches.output.value };
             }
-        }
+
+            // No devices match the query, so just use the default devices from the current type
+            return SearchResult { currentDeviceType, {}, {} };
+        }();
+
+        currentDeviceType = result.type;
+        setup.inputDeviceName = result.input;
+        setup.outputDeviceName = result.output;
     }
 
     insertDefaultDeviceNames (setup);
@@ -327,7 +428,7 @@ String AudioDeviceManager::initialiseFromXML (const XmlElement& xml,
     midiDeviceInfosFromXml.clear();
     enabledMidiInputs.clear();
 
-    forEachXmlChildElementWithTagName (xml, c, "MIDIINPUT")
+    for (auto* c : xml.getChildWithTagNameIterator ("MIDIINPUT"))
         midiDeviceInfosFromXml.add ({ c->getStringAttribute ("name"), c->getStringAttribute ("identifier") });
 
     auto isIdentifierAvailable = [] (const Array<MidiDeviceInfo>& available, const String& identifier)
@@ -398,11 +499,14 @@ void AudioDeviceManager::insertDefaultDeviceNames (AudioDeviceSetup& setup) cons
 {
     if (auto* type = getCurrentDeviceTypeObject())
     {
-        if (numOutputChansNeeded > 0 && setup.outputDeviceName.isEmpty())
-            setup.outputDeviceName = type->getDeviceNames (false) [type->getDefaultDeviceIndex (false)];
+        for (const auto isInput : { false, true })
+        {
+            const auto numChannelsNeeded = isInput ? numInputChansNeeded : numOutputChansNeeded;
+            const auto info = getSetupInfo (setup, isInput);
 
-        if (numInputChansNeeded > 0 && setup.inputDeviceName.isEmpty())
-            setup.inputDeviceName = type->getDeviceNames (true) [type->getDefaultDeviceIndex (true)];
+            if (numChannelsNeeded > 0 && info.name.isEmpty())
+                info.name = type->getDeviceNames (isInput) [type->getDefaultDeviceIndex (isInput)];
+        }
     }
 }
 
@@ -506,7 +610,7 @@ AudioIODeviceType* AudioDeviceManager::getCurrentDeviceTypeObject() const
 
 static void updateSetupChannels (AudioDeviceManager::AudioDeviceSetup& setup, int defaultNumIns, int defaultNumOuts)
 {
-    auto updateChannels = [](const String& deviceName, BigInteger& channels, int defaultNumChannels)
+    auto updateChannels = [] (const String& deviceName, BigInteger& channels, int defaultNumChannels)
     {
         if (deviceName.isEmpty())
         {
@@ -533,6 +637,8 @@ String AudioDeviceManager::setAudioDeviceSetup (const AudioDeviceSetup& newSetup
     else if (currentAudioDevice != nullptr)
         return {};
 
+    stopDevice();
+
     if (getCurrentDeviceTypeObject() == nullptr
         || (newSetup.inputDeviceName.isEmpty() && newSetup.outputDeviceName.isEmpty()))
     {
@@ -544,24 +650,26 @@ String AudioDeviceManager::setAudioDeviceSetup (const AudioDeviceSetup& newSetup
         return {};
     }
 
-    stopDevice();
-
     String error;
 
-    if (currentSetup.inputDeviceName  != newSetup.inputDeviceName
-     || currentSetup.outputDeviceName != newSetup.outputDeviceName
-     || currentAudioDevice == nullptr)
+    const auto needsNewDevice = currentSetup.inputDeviceName  != newSetup.inputDeviceName
+                             || currentSetup.outputDeviceName != newSetup.outputDeviceName
+                             || currentAudioDevice == nullptr;
+
+    if (needsNewDevice)
     {
         deleteCurrentDevice();
         scanDevicesIfNeeded();
 
         auto* type = getCurrentDeviceTypeObject();
 
-        if (newSetup.outputDeviceName.isNotEmpty() && ! deviceListContains (type, false, newSetup.outputDeviceName))
-            return "No such device: " + newSetup.outputDeviceName;
+        for (const auto isInput : { false, true })
+        {
+            const auto name = getSetupInfo (newSetup, isInput).name;
 
-        if (newSetup.inputDeviceName.isNotEmpty() && ! deviceListContains (type, true, newSetup.inputDeviceName))
-            return "No such device: " + newSetup.inputDeviceName;
+            if (name.isNotEmpty() && ! deviceListContains (type, isInput, name))
+                return "No such device: " + name;
+        }
 
         currentAudioDevice.reset (type->createDevice (newSetup.outputDeviceName, newSetup.inputDeviceName));
 
@@ -692,7 +800,7 @@ void AudioDeviceManager::restartLastAudioDevice()
         {
             // This method will only reload the last device that was running
             // before closeAudioDevice() was called - you need to actually open
-            // one first, with setAudioDevice().
+            // one first, with setAudioDeviceSetup().
             jassertfalse;
             return;
         }
@@ -801,11 +909,10 @@ void AudioDeviceManager::audioDeviceIOCallbackInt (const float** inputChannelDat
     const ScopedLock sl (audioCallbackLock);
 
     inputLevelGetter->updateLevel (inputChannelData, numInputChannels, numSamples);
-    outputLevelGetter->updateLevel (const_cast<const float**> (outputChannelData), numOutputChannels, numSamples);
 
     if (callbacks.size() > 0)
     {
-        AudioProcessLoadMeasurer::ScopedTimer timer (loadMeasurer);
+        AudioProcessLoadMeasurer::ScopedTimer timer (loadMeasurer, numSamples);
 
         tempBuffer.setSize (jmax (1, numOutputChannels), jmax (1, numSamples), false, false, true);
 
@@ -840,14 +947,17 @@ void AudioDeviceManager::audioDeviceIOCallbackInt (const float** inputChannelDat
         auto* src = testSound->getReadPointer (0, testSoundPosition);
 
         for (int i = 0; i < numOutputChannels; ++i)
-            for (int j = 0; j < numSamps; ++j)
-                outputChannelData [i][j] += src[j];
+            if (auto* dst = outputChannelData [i])
+                for (int j = 0; j < numSamps; ++j)
+                    dst[j] += src[j];
 
         testSoundPosition += numSamps;
 
         if (testSoundPosition >= testSound->getNumSamples())
             testSound.reset();
     }
+
+    outputLevelGetter->updateLevel (const_cast<const float**> (outputChannelData), numOutputChannels, numSamples);
 }
 
 void AudioDeviceManager::audioDeviceAboutToStartInt (AudioIODevice* const device)
@@ -862,6 +972,7 @@ void AudioDeviceManager::audioDeviceAboutToStartInt (AudioIODevice* const device
             callbacks.getUnchecked(i)->audioDeviceAboutToStart (device);
     }
 
+    updateCurrentSetup();
     sendChangeMessage();
 }
 
@@ -1134,12 +1245,19 @@ void AudioDeviceManager::addMidiInputCallback (const String& name, MidiInputCall
 
 void AudioDeviceManager::removeMidiInputCallback (const String& name, MidiInputCallback* callbackToRemove)
 {
-    for (auto& device : MidiInput::getAvailableDevices())
+    if (name.isEmpty())
     {
-        if (device.name == name)
+        removeMidiInputDeviceCallback ({}, callbackToRemove);
+    }
+    else
+    {
+        for (auto& device : MidiInput::getAvailableDevices())
         {
-            removeMidiInputDeviceCallback (device.identifier, callbackToRemove);
-            return;
+            if (device.name == name)
+            {
+                removeMidiInputDeviceCallback (device.identifier, callbackToRemove);
+                return;
+            }
         }
     }
 }
@@ -1156,4 +1274,456 @@ void AudioDeviceManager::setDefaultMidiOutput (const String& name)
     }
 }
 
+//==============================================================================
+//==============================================================================
+#if JUCE_UNIT_TESTS
+
+class AudioDeviceManagerTests : public UnitTest
+{
+public:
+    AudioDeviceManagerTests() : UnitTest ("AudioDeviceManager", UnitTestCategories::audio) {}
+
+    void runTest() override
+    {
+        beginTest ("When the AudioDeviceSetup has non-empty device names, initialise uses the requested devices");
+        {
+            AudioDeviceManager manager;
+            initialiseManager (manager);
+
+            expectEquals (manager.getAvailableDeviceTypes().size(), 2);
+
+            AudioDeviceManager::AudioDeviceSetup setup;
+            setup.outputDeviceName = "z";
+            setup.inputDeviceName = "c";
+
+            expect (manager.initialise (2, 2, nullptr, true, String{}, &setup).isEmpty());
+
+            const auto& newSetup = manager.getAudioDeviceSetup();
+
+            expectEquals (newSetup.outputDeviceName, setup.outputDeviceName);
+            expectEquals (newSetup.inputDeviceName, setup.inputDeviceName);
+
+            expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+            expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 2);
+        }
+
+        beginTest ("When the AudioDeviceSetup has empty device names, initialise picks suitable default devices");
+        {
+            AudioDeviceManager manager;
+            initialiseManager (manager);
+
+            AudioDeviceManager::AudioDeviceSetup setup;
+
+            expect (manager.initialise (2, 2, nullptr, true, String{}, &setup).isEmpty());
+
+            const auto& newSetup = manager.getAudioDeviceSetup();
+
+            expectEquals (newSetup.outputDeviceName, String ("x"));
+            expectEquals (newSetup.inputDeviceName, String ("a"));
+
+            expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+            expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 2);
+        }
+
+        beginTest ("When the preferred device name matches an input and an output on the same type, that type is used");
+        {
+            AudioDeviceManager manager;
+            initialiseManagerWithDifferentDeviceNames (manager);
+
+            expect (manager.initialise (2, 2, nullptr, true, "bar *").isEmpty());
+
+            expectEquals (manager.getCurrentAudioDeviceType(), String ("bar"));
+
+            const auto& newSetup = manager.getAudioDeviceSetup();
+
+            expectEquals (newSetup.outputDeviceName, String ("bar out a"));
+            expectEquals (newSetup.inputDeviceName, String ("bar in a"));
+
+            expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+            expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 2);
+
+            expect (manager.getCurrentAudioDevice() != nullptr);
+        }
+
+        beginTest ("When the preferred device name matches either an input and an output, but not both, that type is used");
+        {
+            AudioDeviceManager manager;
+            initialiseManagerWithDifferentDeviceNames (manager);
+
+            expect (manager.initialise (2, 2, nullptr, true, "bar out b").isEmpty());
+
+            expectEquals (manager.getCurrentAudioDeviceType(), String ("bar"));
+
+            const auto& newSetup = manager.getAudioDeviceSetup();
+
+            expectEquals (newSetup.outputDeviceName, String ("bar out b"));
+            expectEquals (newSetup.inputDeviceName, String ("bar in a"));
+
+            expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+            expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 2);
+
+            expect (manager.getCurrentAudioDevice() != nullptr);
+        }
+
+        beginTest ("When the preferred device name does not match any inputs or outputs, defaults are used");
+        {
+            AudioDeviceManager manager;
+            initialiseManagerWithDifferentDeviceNames (manager);
+
+            expect (manager.initialise (2, 2, nullptr, true, "unmatchable").isEmpty());
+
+            expectEquals (manager.getCurrentAudioDeviceType(), String ("foo"));
+
+            const auto& newSetup = manager.getAudioDeviceSetup();
+
+            expectEquals (newSetup.outputDeviceName, String ("foo out a"));
+            expectEquals (newSetup.inputDeviceName, String ("foo in a"));
+
+            expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+            expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 2);
+
+            expect (manager.getCurrentAudioDevice() != nullptr);
+        }
+
+        beginTest ("When first device type has no devices, a device type with devices is used instead");
+        {
+            AudioDeviceManager manager;
+            initialiseManagerWithEmptyDeviceType (manager);
+
+            AudioDeviceManager::AudioDeviceSetup setup;
+
+            expect (manager.initialise (2, 2, nullptr, true, {}, &setup).isEmpty());
+
+            const auto& newSetup = manager.getAudioDeviceSetup();
+
+            expectEquals (newSetup.outputDeviceName, String ("x"));
+            expectEquals (newSetup.inputDeviceName, String ("a"));
+
+            expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+            expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 2);
+        }
+
+        beginTest ("If a device type has been explicitly set to a type with devices, "
+                   "initialisation should respect this choice");
+        {
+            AudioDeviceManager manager;
+            initialiseManagerWithEmptyDeviceType (manager);
+            manager.setCurrentAudioDeviceType (mockBName, true);
+
+            AudioDeviceManager::AudioDeviceSetup setup;
+            expect (manager.initialise (2, 2, nullptr, true, {}, &setup).isEmpty());
+
+            expectEquals (manager.getCurrentAudioDeviceType(), mockBName);
+        }
+
+        beginTest ("If a device type has been explicitly set to a type without devices, "
+                   "initialisation should pick a type with devices instead");
+        {
+            AudioDeviceManager manager;
+            initialiseManagerWithEmptyDeviceType (manager);
+            manager.setCurrentAudioDeviceType (emptyName, true);
+
+            AudioDeviceManager::AudioDeviceSetup setup;
+            expect (manager.initialise (2, 2, nullptr, true, {}, &setup).isEmpty());
+
+            expectEquals (manager.getCurrentAudioDeviceType(), mockAName);
+        }
+
+        beginTest ("Carry out a long sequence of configuration changes");
+        {
+            AudioDeviceManager manager;
+            initialiseManagerWithEmptyDeviceType    (manager);
+            initialiseWithDefaultDevices            (manager);
+            disableInputChannelsButLeaveDeviceOpen  (manager);
+            selectANewInputDevice                   (manager);
+            disableInputDevice                      (manager);
+            reenableInputDeviceWithNoChannels       (manager);
+            enableInputChannels                     (manager);
+            disableInputChannelsButLeaveDeviceOpen  (manager);
+            switchDeviceType                        (manager);
+            enableInputChannels                     (manager);
+            closeDeviceByRequestingEmptyNames       (manager);
+        }
+    }
+
+private:
+    void initialiseWithDefaultDevices (AudioDeviceManager& manager)
+    {
+        manager.initialiseWithDefaultDevices (2, 2);
+        const auto& setup = manager.getAudioDeviceSetup();
+
+        expectEquals (setup.inputChannels.countNumberOfSetBits(), 2);
+        expectEquals (setup.outputChannels.countNumberOfSetBits(), 2);
+
+        expect (setup.useDefaultInputChannels);
+        expect (setup.useDefaultOutputChannels);
+
+        expect (manager.getCurrentAudioDevice() != nullptr);
+    }
+
+    void disableInputChannelsButLeaveDeviceOpen (AudioDeviceManager& manager)
+    {
+        auto setup = manager.getAudioDeviceSetup();
+        setup.inputChannels.clear();
+        setup.useDefaultInputChannels = false;
+
+        expect (manager.setAudioDeviceSetup (setup, true).isEmpty());
+
+        const auto newSetup = manager.getAudioDeviceSetup();
+        expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 0);
+        expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+
+        expect (! newSetup.useDefaultInputChannels);
+        expect (newSetup.useDefaultOutputChannels);
+
+        expectEquals (newSetup.inputDeviceName, setup.inputDeviceName);
+        expectEquals (newSetup.outputDeviceName, setup.outputDeviceName);
+
+        expect (manager.getCurrentAudioDevice() != nullptr);
+    }
+
+    void selectANewInputDevice (AudioDeviceManager& manager)
+    {
+        auto setup = manager.getAudioDeviceSetup();
+        setup.inputDeviceName = "b";
+
+        expect (manager.setAudioDeviceSetup (setup, true).isEmpty());
+
+        const auto newSetup = manager.getAudioDeviceSetup();
+        expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 0);
+        expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+
+        expect (! newSetup.useDefaultInputChannels);
+        expect (newSetup.useDefaultOutputChannels);
+
+        expectEquals (newSetup.inputDeviceName, setup.inputDeviceName);
+        expectEquals (newSetup.outputDeviceName, setup.outputDeviceName);
+
+        expect (manager.getCurrentAudioDevice() != nullptr);
+    }
+
+    void disableInputDevice (AudioDeviceManager& manager)
+    {
+        auto setup = manager.getAudioDeviceSetup();
+        setup.inputDeviceName = "";
+
+        expect (manager.setAudioDeviceSetup (setup, true).isEmpty());
+
+        const auto newSetup = manager.getAudioDeviceSetup();
+        expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 0);
+        expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+
+        expect (! newSetup.useDefaultInputChannels);
+        expect (newSetup.useDefaultOutputChannels);
+
+        expectEquals (newSetup.inputDeviceName, setup.inputDeviceName);
+        expectEquals (newSetup.outputDeviceName, setup.outputDeviceName);
+
+        expect (manager.getCurrentAudioDevice() != nullptr);
+    }
+
+    void reenableInputDeviceWithNoChannels (AudioDeviceManager& manager)
+    {
+        auto setup = manager.getAudioDeviceSetup();
+        setup.inputDeviceName = "a";
+
+        expect (manager.setAudioDeviceSetup (setup, true).isEmpty());
+
+        const auto newSetup = manager.getAudioDeviceSetup();
+        expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 0);
+        expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+
+        expect (! newSetup.useDefaultInputChannels);
+        expect (newSetup.useDefaultOutputChannels);
+
+        expectEquals (newSetup.inputDeviceName, setup.inputDeviceName);
+        expectEquals (newSetup.outputDeviceName, setup.outputDeviceName);
+
+        expect (manager.getCurrentAudioDevice() != nullptr);
+    }
+
+    void enableInputChannels (AudioDeviceManager& manager)
+    {
+        auto setup = manager.getAudioDeviceSetup();
+        setup.inputDeviceName = manager.getCurrentDeviceTypeObject()->getDeviceNames (true)[0];
+        setup.inputChannels = 3;
+        setup.useDefaultInputChannels = false;
+
+        expect (manager.setAudioDeviceSetup (setup, true).isEmpty());
+
+        const auto newSetup = manager.getAudioDeviceSetup();
+        expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 2);
+        expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+
+        expect (! newSetup.useDefaultInputChannels);
+        expect (newSetup.useDefaultOutputChannels);
+
+        expectEquals (newSetup.inputDeviceName, setup.inputDeviceName);
+        expectEquals (newSetup.outputDeviceName, setup.outputDeviceName);
+
+        expect (manager.getCurrentAudioDevice() != nullptr);
+    }
+
+    void switchDeviceType (AudioDeviceManager& manager)
+    {
+        const auto oldSetup = manager.getAudioDeviceSetup();
+
+        expectEquals (manager.getCurrentAudioDeviceType(), String (mockAName));
+
+        manager.setCurrentAudioDeviceType (mockBName, true);
+
+        expectEquals (manager.getCurrentAudioDeviceType(), String (mockBName));
+
+        const auto newSetup = manager.getAudioDeviceSetup();
+
+        expect (newSetup.outputDeviceName.isNotEmpty());
+        // We had no channels enabled, which means we don't need to open a new input device
+        expect (newSetup.inputDeviceName.isEmpty());
+
+        expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 0);
+        expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+
+        expect (manager.getCurrentAudioDevice() != nullptr);
+    }
+
+    void closeDeviceByRequestingEmptyNames (AudioDeviceManager& manager)
+    {
+        auto setup = manager.getAudioDeviceSetup();
+        setup.inputDeviceName = "";
+        setup.outputDeviceName = "";
+
+        expect (manager.setAudioDeviceSetup (setup, true).isEmpty());
+
+        const auto newSetup = manager.getAudioDeviceSetup();
+        expectEquals (newSetup.inputChannels.countNumberOfSetBits(), 2);
+        expectEquals (newSetup.outputChannels.countNumberOfSetBits(), 2);
+
+        expect (newSetup.inputDeviceName.isEmpty());
+        expect (newSetup.outputDeviceName.isEmpty());
+
+        expect (manager.getCurrentAudioDevice() == nullptr);
+    }
+
+    const String mockAName = "mockA";
+    const String mockBName = "mockB";
+    const String emptyName = "empty";
+
+    class MockDevice : public AudioIODevice
+    {
+    public:
+        MockDevice (String typeNameIn, String outNameIn, String inNameIn)
+            : AudioIODevice ("mock", typeNameIn), outName (outNameIn), inName (inNameIn) {}
+
+        StringArray getOutputChannelNames() override { return { "o1", "o2", "o3" }; }
+        StringArray getInputChannelNames()  override { return { "i1", "i2", "i3" }; }
+
+        Array<double> getAvailableSampleRates() override { return { 44100.0, 48000.0 }; }
+        Array<int> getAvailableBufferSizes() override { return { 128, 256 }; }
+        int getDefaultBufferSize() override { return 128; }
+
+        String open (const BigInteger& inputs, const BigInteger& outputs, double sr, int bs) override
+        {
+            inChannels = inputs;
+            outChannels = outputs;
+            sampleRate = sr;
+            blockSize = bs;
+            on = true;
+            return {};
+        }
+
+        void close() override { on = false; }
+        bool isOpen() override { return on; }
+
+        void start (AudioIODeviceCallback*) override { playing = true; }
+        void stop() override { playing = false; }
+        bool isPlaying() override { return playing; }
+
+        String getLastError() override { return {}; }
+        int getCurrentBufferSizeSamples() override { return blockSize; }
+        double getCurrentSampleRate() override { return sampleRate; }
+        int getCurrentBitDepth() override { return 16; }
+
+        BigInteger getActiveOutputChannels() const override { return outChannels; }
+        BigInteger getActiveInputChannels()  const override { return inChannels; }
+
+        int getOutputLatencyInSamples() override { return 0; }
+        int getInputLatencyInSamples() override { return 0; }
+
+    private:
+        String outName, inName;
+        BigInteger outChannels, inChannels;
+        double sampleRate = 0.0;
+        int blockSize = 0;
+        bool on = false, playing = false;
+    };
+
+    class MockDeviceType : public AudioIODeviceType
+    {
+    public:
+        explicit MockDeviceType (String kind)
+            : MockDeviceType (std::move (kind), { "a", "b", "c" }, { "x", "y", "z" }) {}
+
+        MockDeviceType (String kind, StringArray inputNames, StringArray outputNames)
+            : AudioIODeviceType (std::move (kind)),
+              inNames (std::move (inputNames)),
+              outNames (std::move (outputNames)) {}
+
+        void scanForDevices() override {}
+
+        StringArray getDeviceNames (bool isInput) const override
+        {
+            return getNames (isInput);
+        }
+
+        int getDefaultDeviceIndex (bool) const override { return 0; }
+
+        int getIndexOfDevice (AudioIODevice* device, bool isInput) const override
+        {
+            return getNames (isInput).indexOf (device->getName());
+        }
+
+        bool hasSeparateInputsAndOutputs() const override { return true; }
+
+        AudioIODevice* createDevice (const String& outputName, const String& inputName) override
+        {
+            if (inNames.contains (inputName) || outNames.contains (outputName))
+                return new MockDevice (getTypeName(), outputName, inputName);
+
+            return nullptr;
+        }
+
+    private:
+        const StringArray& getNames (bool isInput) const { return isInput ? inNames : outNames; }
+
+        const StringArray inNames, outNames;
+    };
+
+    void initialiseManager (AudioDeviceManager& manager)
+    {
+        manager.addAudioDeviceType (std::make_unique<MockDeviceType> (mockAName));
+        manager.addAudioDeviceType (std::make_unique<MockDeviceType> (mockBName));
+    }
+
+    void initialiseManagerWithEmptyDeviceType (AudioDeviceManager& manager)
+    {
+        manager.addAudioDeviceType (std::make_unique<MockDeviceType> (emptyName, StringArray{}, StringArray{}));
+        initialiseManager (manager);
+    }
+
+    void initialiseManagerWithDifferentDeviceNames (AudioDeviceManager& manager)
+    {
+        manager.addAudioDeviceType (std::make_unique<MockDeviceType> ("foo",
+                                                                      StringArray { "foo in a", "foo in b" },
+                                                                      StringArray { "foo out a", "foo out b" }));
+
+        manager.addAudioDeviceType (std::make_unique<MockDeviceType> ("bar",
+                                                                      StringArray { "bar in a", "bar in b" },
+                                                                      StringArray { "bar out a", "bar out b" }));
+    }
+};
+
+static AudioDeviceManagerTests audioDeviceManagerTests;
+
+#endif
+
 } // namespace juce
