@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -44,8 +43,8 @@ struct AudioUnitHelpers
 
         void alloc()
         {
-            const int numInputBuses  = AudioUnitHelpers::getBusCount (&processor, true);
-            const int numOutputBuses = AudioUnitHelpers::getBusCount (&processor, false);
+            const int numInputBuses  = AudioUnitHelpers::getBusCount (processor, true);
+            const int numOutputBuses = AudioUnitHelpers::getBusCount (processor, false);
 
             initializeChannelMapArray (true, numInputBuses);
             initializeChannelMapArray (false, numOutputBuses);
@@ -66,7 +65,7 @@ struct AudioUnitHelpers
             outputLayoutMapStorage.free();
         }
 
-        inline const int* get (bool input, int bus) const noexcept { return (input ? inputLayoutMap : outputLayoutMap) [bus]; }
+        inline const int* get (bool input, int bus) const noexcept { return (input ? inputLayoutMap : outputLayoutMap)[bus]; }
 
     private:
         //==============================================================================
@@ -120,15 +119,24 @@ struct AudioUnitHelpers
     class CoreAudioBufferList
     {
     public:
-        CoreAudioBufferList() { reset(); }
-
-        //==============================================================================
-        void prepare (int inChannels, int outChannels, int maxFrames)
+        void prepare (const AudioProcessor::BusesLayout& layout, int maxFrames)
         {
-            const int numChannels = jmax (inChannels, outChannels);
+            const auto getChannelOffsets = [] (const auto& range)
+            {
+                std::vector<int> result { 0 };
 
+                for (const auto& bus : range)
+                    result.push_back (result.back() + bus.size());
+
+                return result;
+            };
+
+            inputBusOffsets  = getChannelOffsets (layout.inputBuses);
+            outputBusOffsets = getChannelOffsets (layout.outputBuses);
+
+            const auto numChannels = jmax (inputBusOffsets.back(), outputBusOffsets.back());
             scratch.setSize (numChannels, maxFrames);
-            channels.calloc (static_cast<size_t> (numChannels));
+            channels.resize (static_cast<size_t> (numChannels), nullptr);
 
             reset();
         }
@@ -136,106 +144,107 @@ struct AudioUnitHelpers
         void release()
         {
             scratch.setSize (0, 0);
-            channels.free();
+            channels = {};
+            inputBusOffsets = outputBusOffsets = std::vector<int>();
         }
 
-        void reset() noexcept
+        void reset()
         {
-            pushIdx = 0;
-            popIdx = 0;
-            zeromem (channels.get(), sizeof(float*) * static_cast<size_t> (scratch.getNumChannels()));
+            std::fill (channels.begin(), channels.end(), nullptr);
         }
 
-        //==============================================================================
         float* setBuffer (const int idx, float* ptr = nullptr) noexcept
         {
             jassert (idx < scratch.getNumChannels());
-            return (channels [idx] = uniqueBuffer (idx, ptr));
+            return channels[(size_t) idx] = uniqueBuffer (idx, ptr);
         }
 
-        //==============================================================================
-        float* push() noexcept
-        {
-            jassert (pushIdx < scratch.getNumChannels());
-            return channels [pushIdx++];
-        }
-
-        void push (AudioBufferList& bufferList, const int* channelMap) noexcept
-        {
-            jassert (pushIdx < scratch.getNumChannels());
-
-            if (bufferList.mNumberBuffers > 0)
-            {
-                const UInt32 n = bufferList.mBuffers [0].mDataByteSize /
-                (bufferList.mBuffers [0].mNumberChannels * sizeof (float));
-                const bool isInterleaved = isAudioBufferInterleaved (bufferList);
-                const int numChannels = static_cast<int> (isInterleaved ? bufferList.mBuffers [0].mNumberChannels
-                                                          : bufferList.mNumberBuffers);
-
-                for (int ch = 0; ch < numChannels; ++ch)
-                {
-                    float* data = push();
-
-                    int mappedChannel = channelMap [ch];
-                    if (isInterleaved || static_cast<float*> (bufferList.mBuffers [mappedChannel].mData) != data)
-                        copyAudioBuffer (bufferList, mappedChannel, n, data);
-                }
-            }
-        }
-
-        //==============================================================================
-        float* pop() noexcept
-        {
-            jassert (popIdx < scratch.getNumChannels());
-            return channels[popIdx++];
-        }
-
-        void pop (AudioBufferList& buffer, const int* channelMap) noexcept
-        {
-            if (buffer.mNumberBuffers > 0)
-            {
-                const UInt32 n = buffer.mBuffers [0].mDataByteSize / (buffer.mBuffers [0].mNumberChannels * sizeof (float));
-                const bool isInterleaved = isAudioBufferInterleaved (buffer);
-                const int numChannels = static_cast<int> (isInterleaved ? buffer.mBuffers [0].mNumberChannels : buffer.mNumberBuffers);
-
-                for (int ch = 0; ch < numChannels; ++ch)
-                {
-                    int mappedChannel = channelMap [ch];
-                    float* nextBuffer = pop();
-
-                    if (nextBuffer == buffer.mBuffers [mappedChannel].mData && ! isInterleaved)
-                        continue; // no copying necessary
-
-                    if (buffer.mBuffers [mappedChannel].mData == nullptr && ! isInterleaved)
-                        buffer.mBuffers [mappedChannel].mData = nextBuffer;
-                    else
-                        copyAudioBuffer (nextBuffer, mappedChannel, n, buffer);
-                }
-            }
-        }
-
-        //==============================================================================
         AudioBuffer<float>& getBuffer (UInt32 frames) noexcept
         {
-            jassert (pushIdx == scratch.getNumChannels());
+           #if JUCE_DEBUG
+            for (int i = 0; i < (int) channels.size(); ++i)
+                jassert (channels[(size_t) i] != nullptr);
+           #endif
 
-          #if JUCE_DEBUG
-            for (int i = 0; i < pushIdx; ++i)
-                jassert (channels [i] != nullptr);
-          #endif
-
-            mutableBuffer.setDataToReferTo (channels, pushIdx, static_cast<int> (frames));
+            mutableBuffer.setDataToReferTo (channels.data(), (int) channels.size(), static_cast<int> (frames));
             return mutableBuffer;
         }
 
+        void set (int bus, AudioBufferList& bufferList, const int* channelMap) noexcept
+        {
+            if (bufferList.mNumberBuffers <= 0 || ! isPositiveAndBelow (bus, inputBusOffsets.size() - 1))
+                return;
+
+            const auto n = (UInt32) (bufferList.mBuffers[0].mDataByteSize / (bufferList.mBuffers[0].mNumberChannels * sizeof (float)));
+            const auto isInterleaved = isAudioBufferInterleaved (bufferList);
+            const auto numChannels = (int) (isInterleaved ? bufferList.mBuffers[0].mNumberChannels
+                                                          : bufferList.mNumberBuffers);
+
+            for (int ch = 0; ch < numChannels; ++ch)
+            {
+                float* data = channels[(size_t) (inputBusOffsets[(size_t) bus] + ch)];
+
+                const auto mappedChannel = channelMap[ch];
+
+                if (isInterleaved || static_cast<float*> (bufferList.mBuffers[mappedChannel].mData) != data)
+                    copyAudioBuffer (bufferList, mappedChannel, n, data);
+            }
+        }
+
+        void get (int bus, AudioBufferList& buffer, const int* channelMap) noexcept
+        {
+            if (buffer.mNumberBuffers <= 0 || ! isPositiveAndBelow (bus, outputBusOffsets.size() - 1))
+                return;
+
+            const auto n = (UInt32) (buffer.mBuffers[0].mDataByteSize / (buffer.mBuffers[0].mNumberChannels * sizeof (float)));
+            const auto isInterleaved = isAudioBufferInterleaved (buffer);
+            const auto numChannels = (int) (isInterleaved ? buffer.mBuffers[0].mNumberChannels
+                                                          : buffer.mNumberBuffers);
+
+            for (int ch = 0; ch < numChannels; ++ch)
+            {
+                float* data = channels[(size_t) (outputBusOffsets[(size_t) bus] + ch)];
+
+                const auto mappedChannel = channelMap[ch];
+
+                if (data == buffer.mBuffers[mappedChannel].mData && ! isInterleaved)
+                    continue; // no copying necessary
+
+                if (buffer.mBuffers[mappedChannel].mData == nullptr && ! isInterleaved)
+                    buffer.mBuffers[mappedChannel].mData = data;
+                else
+                    copyAudioBuffer (data, mappedChannel, n, buffer);
+            }
+        }
+
+        void clearInputBus (int index)
+        {
+            if (isPositiveAndBelow (index, inputBusOffsets.size() - 1))
+                clearChannels (inputBusOffsets[(size_t) index], inputBusOffsets[(size_t) (index + 1)]);
+        }
+
+        void clearUnusedChannels()
+        {
+            jassert (! inputBusOffsets .empty());
+            jassert (! outputBusOffsets.empty());
+
+            clearChannels (inputBusOffsets.back(), outputBusOffsets.back());
+        }
+
     private:
+        void clearChannels (int begin, int end)
+        {
+            for (auto i = begin; i < end; ++i)
+                zeromem (scratch.getWritePointer (i), sizeof (float) * (size_t) scratch.getNumSamples());
+        }
+
         float* uniqueBuffer (int idx, float* buffer) noexcept
         {
             if (buffer == nullptr)
                 return scratch.getWritePointer (idx);
 
             for (int ch = 0; ch < idx; ++ch)
-                if (buffer == channels[ch])
+                if (buffer == channels[(size_t) ch])
                     return scratch.getWritePointer (idx);
 
             return buffer;
@@ -243,8 +252,8 @@ struct AudioUnitHelpers
 
         //==============================================================================
         AudioBuffer<float> scratch, mutableBuffer;
-        HeapBlock<float*> channels;
-        int pushIdx, popIdx;
+        std::vector<float*> channels;
+        std::vector<int> inputBusOffsets, outputBusOffsets;
     };
 
     static bool isAudioBufferInterleaved (const AudioBufferList& audioBuffer) noexcept
@@ -308,7 +317,7 @@ struct AudioUnitHelpers
     static bool isLayoutSupported (const AudioProcessor& processor,
                                    bool isInput, int busIdx,
                                    int numChannels,
-                                   const short (&channelLayoutList) [numLayouts][2],
+                                   const short (&channelLayoutList)[numLayouts][2],
                                    bool hasLayoutMap = true)
     {
         if (const AudioProcessor::Bus* bus = processor.getBus (isInput, busIdx))
@@ -335,8 +344,8 @@ struct AudioUnitHelpers
     {
         Array<AUChannelInfo> channelInfo;
 
-        auto hasMainInputBus  = (AudioUnitHelpers::getBusCount (&processor, true)  > 0);
-        auto hasMainOutputBus = (AudioUnitHelpers::getBusCount (&processor, false) > 0);
+        auto hasMainInputBus  = (AudioUnitHelpers::getBusCountForWrapper (processor, true)  > 0);
+        auto hasMainOutputBus = (AudioUnitHelpers::getBusCountForWrapper (processor, false) > 0);
 
         if ((! hasMainInputBus) && (! hasMainOutputBus))
         {
@@ -354,11 +363,22 @@ struct AudioUnitHelpers
         auto defaultInputs  = processor.getChannelCountOfBus (true,  0);
         auto defaultOutputs = processor.getChannelCountOfBus (false, 0);
 
-        SortedSet<int> supportedChannels;
+        struct Channels
+        {
+            SInt16 ins, outs;
+
+            std::pair<SInt16, SInt16> makePair() const noexcept { return std::make_pair (ins, outs); }
+
+            bool operator<  (const Channels& other) const noexcept { return makePair() <  other.makePair(); }
+            bool operator== (const Channels& other) const noexcept { return makePair() == other.makePair(); }
+        };
+
+        SortedSet<Channels> supportedChannels;
 
         // add the current configuration
         if (defaultInputs != 0 || defaultOutputs != 0)
-            supportedChannels.add ((defaultInputs << 16) | defaultOutputs);
+            supportedChannels.add ({ static_cast<SInt16> (defaultInputs),
+                                     static_cast<SInt16> (defaultOutputs) });
 
         for (auto inChanNum = hasMainInputBus ? 1 : 0; inChanNum <= (hasMainInputBus ? maxNumChanToCheckFor : 0); ++inChanNum)
         {
@@ -376,19 +396,16 @@ struct AudioUnitHelpers
                     if (! isNumberOfChannelsSupported (outBus, outChanNum, outLayout))
                         continue;
 
-                supportedChannels.add (((hasMainInputBus  ? outLayout.getMainInputChannels()  : 0) << 16)
-                                      | (hasMainOutputBus ? outLayout.getMainOutputChannels() : 0));
+                supportedChannels.add ({ static_cast<SInt16> (hasMainInputBus  ? outLayout.getMainInputChannels()  : 0),
+                                         static_cast<SInt16> (hasMainOutputBus ? outLayout.getMainOutputChannels() : 0) });
             }
         }
 
         auto hasInOutMismatch = false;
 
-        for (auto supported : supportedChannels)
+        for (const auto& supported : supportedChannels)
         {
-            auto numInputs  = (supported >> 16) & 0xffff;
-            auto numOutputs = (supported >> 0)  & 0xffff;
-
-            if (numInputs != numOutputs)
+            if (supported.ins != supported.outs)
             {
                 hasInOutMismatch = true;
                 break;
@@ -399,7 +416,8 @@ struct AudioUnitHelpers
 
         for (auto inChanNum = hasMainInputBus ? 1 : 0; inChanNum <= (hasMainInputBus ? maxNumChanToCheckFor : 0); ++inChanNum)
         {
-            auto channelConfiguration = (inChanNum << 16) | (hasInOutMismatch ? defaultOutputs : inChanNum);
+            Channels channelConfiguration { static_cast<SInt16> (inChanNum),
+                                            static_cast<SInt16> (hasInOutMismatch ? defaultOutputs : inChanNum) };
 
             if (! supportedChannels.contains (channelConfiguration))
             {
@@ -410,7 +428,8 @@ struct AudioUnitHelpers
 
         for (auto outChanNum = hasMainOutputBus ? 1 : 0; outChanNum <= (hasMainOutputBus ? maxNumChanToCheckFor : 0); ++outChanNum)
         {
-            auto channelConfiguration = ((hasInOutMismatch ? defaultInputs : outChanNum) << 16) | outChanNum;
+            Channels channelConfiguration { static_cast<SInt16> (hasInOutMismatch ? defaultInputs : outChanNum),
+                                            static_cast<SInt16> (outChanNum) };
 
             if (! supportedChannels.contains (channelConfiguration))
             {
@@ -419,16 +438,13 @@ struct AudioUnitHelpers
             }
         }
 
-        for (auto supported : supportedChannels)
+        for (const auto& supported : supportedChannels)
         {
-            auto numInputs  = (supported >> 16) & 0xffff;
-            auto numOutputs = (supported >> 0)  & 0xffff;
-
             AUChannelInfo info;
 
             // see here: https://developer.apple.com/library/mac/documentation/MusicAudio/Conceptual/AudioUnitProgrammingGuide/TheAudioUnit/TheAudioUnit.html
-            info.inChannels  = static_cast<SInt16> (hasMainInputBus  ? (hasUnsupportedInput  ? numInputs :  (hasInOutMismatch && (! hasUnsupportedOutput) ? -2 : -1)) : 0);
-            info.outChannels = static_cast<SInt16> (hasMainOutputBus ? (hasUnsupportedOutput ? numOutputs : (hasInOutMismatch && (! hasUnsupportedInput)  ? -2 : -1)) : 0);
+            info.inChannels  = static_cast<SInt16> (hasMainInputBus  ? (hasUnsupportedInput  ? supported.ins  : (hasInOutMismatch && (! hasUnsupportedOutput) ? -2 : -1)) : 0);
+            info.outChannels = static_cast<SInt16> (hasMainOutputBus ? (hasUnsupportedOutput ? supported.outs : (hasInOutMismatch && (! hasUnsupportedInput)  ? -2 : -1)) : 0);
 
             if (info.inChannels == -2 && info.outChannels == -2)
                 info.inChannels = -1;
@@ -465,9 +481,9 @@ struct AudioUnitHelpers
     }
 
     //==============================================================================
-    static int getBusCount (const AudioProcessor* juceFilter, bool isInput)
+    static int getBusCount (const AudioProcessor& juceFilter, bool isInput)
     {
-        int busCount = juceFilter->getBusCount (isInput);
+        int busCount = juceFilter.getBusCount (isInput);
 
        #ifdef JucePlugin_PreferredChannelConfigurations
         short configs[][2] = {JucePlugin_PreferredChannelConfigurations};
@@ -485,6 +501,17 @@ struct AudioUnitHelpers
         return busCount;
     }
 
+    static int getBusCountForWrapper (const AudioProcessor& juceFilter, bool isInput)
+    {
+       #if JucePlugin_IsMidiEffect
+        const auto numRequiredBuses = isInput ? 0 : 1;
+       #else
+        const auto numRequiredBuses = 0;
+       #endif
+
+        return jmax (numRequiredBuses, getBusCount (juceFilter, isInput));
+    }
+
     static bool setBusesLayout (AudioProcessor* juceFilter, const AudioProcessor::BusesLayout& requestedLayouts)
     {
        #ifdef JucePlugin_PreferredChannelConfigurations
@@ -495,7 +522,7 @@ struct AudioUnitHelpers
             const bool isInput = (dir == 0);
 
             const int actualBuses = juceFilter->getBusCount (isInput);
-            const int auNumBuses  = getBusCount (juceFilter, isInput);
+            const int auNumBuses  = getBusCount (*juceFilter, isInput);
             Array<AudioChannelSet>& buses = (isInput ? copy.inputBuses : copy.outputBuses);
 
             for (int i = auNumBuses; i < actualBuses; ++i)
@@ -518,7 +545,7 @@ struct AudioUnitHelpers
             const bool isInput = (dir == 0);
 
             const int actualBuses = juceFilter->getBusCount (isInput);
-            const int auNumBuses  = getBusCount (juceFilter, isInput);
+            const int auNumBuses  = getBusCount (*juceFilter, isInput);
             auto& buses = (isInput ? layout.inputBuses : layout.outputBuses);
 
             for (int i = auNumBuses; i < actualBuses; ++i)
@@ -532,6 +559,6 @@ struct AudioUnitHelpers
     }
 };
 
-#endif // ! DOXYGEN
+#endif
 
 } // namespace juce
