@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -196,7 +195,7 @@ void MidiKeyboardComponent::setMidiChannel (int midiChannelNumber)
 void MidiKeyboardComponent::setMidiChannelsToDisplay (int midiChannelMask)
 {
     midiInChannelMask = midiChannelMask;
-    shouldCheckState = true;
+    noPendingUpdates.store (false);
 }
 
 void MidiKeyboardComponent::setVelocity (float v, bool useMousePosition)
@@ -221,7 +220,7 @@ Range<float> MidiKeyboardComponent::getKeyPosition (int midiNoteNumber, float ta
     auto octave = midiNoteNumber / 12;
     auto note   = midiNoteNumber % 12;
 
-    auto start = octave * 7.0f * targetKeyWidth + notePos[note] * targetKeyWidth;
+    auto start = (float) octave * 7.0f * targetKeyWidth + notePos[note] * targetKeyWidth;
     auto width = MidiMessage::isMidiNoteBlack (note) ? blackNoteWidthRatio * targetKeyWidth : targetKeyWidth;
 
     return { start, start + width };
@@ -249,8 +248,8 @@ Rectangle<float> MidiKeyboardComponent::getRectangleForKey (int note) const
         switch (orientation)
         {
             case horizontalKeyboard:            return { x, 0, w, blackNoteLength };
-            case verticalKeyboardFacingLeft:    return { getWidth() - blackNoteLength, x, blackNoteLength, w };
-            case verticalKeyboardFacingRight:   return { 0, getHeight() - x - w, blackNoteLength, w };
+            case verticalKeyboardFacingLeft:    return { (float) getWidth() - blackNoteLength, x, blackNoteLength, w };
+            case verticalKeyboardFacingRight:   return { 0, (float) getHeight() - x - w, blackNoteLength, w };
             default:                            jassertfalse; break;
         }
     }
@@ -260,7 +259,7 @@ Rectangle<float> MidiKeyboardComponent::getRectangleForKey (int note) const
         {
             case horizontalKeyboard:            return { x, 0, w, (float) getHeight() };
             case verticalKeyboardFacingLeft:    return { 0, x, (float) getWidth(), w };
-            case verticalKeyboardFacingRight:   return { 0, getHeight() - x - w, (float) getWidth(), w };
+            case verticalKeyboardFacingRight:   return { 0, (float) getHeight() - x - w, (float) getWidth(), w };
             default:                            jassertfalse; break;
         }
     }
@@ -280,14 +279,13 @@ float MidiKeyboardComponent::getTotalKeyboardWidth() const noexcept
 
 int MidiKeyboardComponent::getNoteAtPosition (Point<float> p)
 {
-    float v;
-    return xyToNote (p, v);
+    return xyToNote (p).note;
 }
 
-int MidiKeyboardComponent::xyToNote (Point<float> pos, float& mousePositionVelocity)
+MidiKeyboardComponent::NoteAndVelocity MidiKeyboardComponent::xyToNote (Point<float> pos)
 {
-    if (! reallyContains (pos.toInt(), false))
-        return -1;
+    if (! reallyContains (pos, false))
+        return { -1, 0.0f };
 
     auto p = pos;
 
@@ -296,15 +294,15 @@ int MidiKeyboardComponent::xyToNote (Point<float> pos, float& mousePositionVeloc
         p = { p.y, p.x };
 
         if (orientation == verticalKeyboardFacingLeft)
-            p = { p.x, getWidth() - p.y };
+            p = { p.x, (float) getWidth() - p.y };
         else
-            p = { getHeight() - p.x, p.y };
+            p = { (float) getHeight() - p.x, p.y };
     }
 
-    return remappedXYToNote (p + Point<float> (xOffset, 0), mousePositionVelocity);
+    return remappedXYToNote (p + Point<float> (xOffset, 0));
 }
 
-int MidiKeyboardComponent::remappedXYToNote (Point<float> pos, float& mousePositionVelocity) const
+MidiKeyboardComponent::NoteAndVelocity MidiKeyboardComponent::remappedXYToNote (Point<float> pos) const
 {
     auto blackNoteLength = getBlackNoteLength();
 
@@ -316,12 +314,11 @@ int MidiKeyboardComponent::remappedXYToNote (Point<float> pos, float& mousePosit
             {
                 auto note = octaveStart + blackNotes[i];
 
-                if (note >= rangeStart && note <= rangeEnd)
+                if (rangeStart <= note && note <= rangeEnd)
                 {
                     if (getKeyPos (note).contains (pos.x - xOffset))
                     {
-                        mousePositionVelocity = jmax (0.0f, pos.y / blackNoteLength);
-                        return note;
+                        return { note, jmax (0.0f, pos.y / blackNoteLength) };
                     }
                 }
             }
@@ -339,15 +336,13 @@ int MidiKeyboardComponent::remappedXYToNote (Point<float> pos, float& mousePosit
                 if (getKeyPos (note).contains (pos.x - xOffset))
                 {
                     auto whiteNoteLength = (orientation == horizontalKeyboard) ? getHeight() : getWidth();
-                    mousePositionVelocity = jmax (0.0f, pos.y / (float) whiteNoteLength);
-                    return note;
+                    return { note, jmax (0.0f, pos.y / (float) whiteNoteLength) };
                 }
             }
         }
     }
 
-    mousePositionVelocity = 0;
-    return -1;
+    return { -1, 0 };
 }
 
 //==============================================================================
@@ -383,8 +378,8 @@ void MidiKeyboardComponent::paint (Graphics& g)
 
     if (orientation == verticalKeyboardFacingLeft)
     {
-        x1 = width - 1.0f;
-        x2 = width - 5.0f;
+        x1 = (float) width - 1.0f;
+        x2 = (float) width - 5.0f;
     }
     else if (orientation == verticalKeyboardFacingRight)
         x2 = 5.0f;
@@ -401,7 +396,7 @@ void MidiKeyboardComponent::paint (Graphics& g)
         switch (orientation)
         {
             case horizontalKeyboard:            g.fillRect (0.0f, 0.0f, x, 5.0f); break;
-            case verticalKeyboardFacingLeft:    g.fillRect (width - 5.0f, 0.0f, 5.0f, x); break;
+            case verticalKeyboardFacingLeft:    g.fillRect ((float) width - 5.0f, 0.0f, 5.0f, x); break;
             case verticalKeyboardFacingRight:   g.fillRect (0.0f, 0.0f, 5.0f, x); break;
             default: break;
         }
@@ -413,9 +408,9 @@ void MidiKeyboardComponent::paint (Graphics& g)
 
         switch (orientation)
         {
-            case horizontalKeyboard:            g.fillRect (0.0f, height - 1.0f, x, 1.0f); break;
+            case horizontalKeyboard:            g.fillRect (0.0f, (float) height - 1.0f, x, 1.0f); break;
             case verticalKeyboardFacingLeft:    g.fillRect (0.0f, 0.0f, 1.0f, x); break;
-            case verticalKeyboardFacingRight:   g.fillRect (width - 1.0f, 0.0f, 1.0f, x); break;
+            case verticalKeyboardFacingRight:   g.fillRect ((float) width - 1.0f, 0.0f, 1.0f, x); break;
             default: break;
         }
     }
@@ -562,7 +557,7 @@ void MidiKeyboardComponent::drawUpDownButton (Graphics& g, int w, int h,
     g.setColour (findColour (upDownButtonArrowColourId)
                   .withAlpha (buttonDown ? 1.0f : (mouseOver ? 0.6f : 0.4f)));
 
-    g.fillPath (path, path.getTransformToScaleToFit (1.0f, 1.0f, w - 2.0f, h - 2.0f, true));
+    g.fillPath (path, path.getTransformToScaleToFit (1.0f, 1.0f, (float) w - 2.0f, (float) h - 2.0f, true));
 }
 
 void MidiKeyboardComponent::setBlackNoteLengthProportion (float ratio) noexcept
@@ -579,7 +574,7 @@ void MidiKeyboardComponent::setBlackNoteLengthProportion (float ratio) noexcept
 float MidiKeyboardComponent::getBlackNoteLength() const noexcept
 {
     auto whiteNoteLength = orientation == horizontalKeyboard ? getHeight() : getWidth();
-    return whiteNoteLength * blackNoteLengthRatio;
+    return (float) whiteNoteLength * blackNoteLengthRatio;
 }
 
 void MidiKeyboardComponent::setBlackNoteWidthProportion (float ratio) noexcept
@@ -609,7 +604,7 @@ void MidiKeyboardComponent::resized()
         {
             auto kx1 = getKeyPos (rangeStart).getStart();
 
-            if (kx2 - kx1 <= w)
+            if (kx2 - kx1 <= (float) w)
             {
                 firstKey = (float) rangeStart;
                 sendChangeMessage();
@@ -644,9 +639,8 @@ void MidiKeyboardComponent::resized()
 
             auto endOfLastKey = getKeyPos (rangeEnd).getEnd();
 
-            float mousePositionVelocity;
             auto spaceAvailable = w;
-            auto lastStartKey = remappedXYToNote ({ endOfLastKey - spaceAvailable, 0 }, mousePositionVelocity) + 1;
+            auto lastStartKey = remappedXYToNote ({ endOfLastKey - (float) spaceAvailable, 0 }).note + 1;
 
             if (lastStartKey >= 0 && ((int) firstKey) > lastStartKey)
             {
@@ -661,7 +655,7 @@ void MidiKeyboardComponent::resized()
             firstKey = (float) rangeStart;
         }
 
-        scrollUp->setVisible (canScroll && getKeyPos (rangeEnd).getStart() > w);
+        scrollUp->setVisible (canScroll && getKeyPos (rangeEnd).getStart() > (float) w);
         repaint();
     }
 }
@@ -669,12 +663,12 @@ void MidiKeyboardComponent::resized()
 //==============================================================================
 void MidiKeyboardComponent::handleNoteOn (MidiKeyboardState*, int /*midiChannel*/, int /*midiNoteNumber*/, float /*velocity*/)
 {
-    shouldCheckState = true; // (probably being called from the audio thread, so avoid blocking in here)
+    noPendingUpdates.store (false);
 }
 
 void MidiKeyboardComponent::handleNoteOff (MidiKeyboardState*, int /*midiChannel*/, int /*midiNoteNumber*/, float /*velocity*/)
 {
-    shouldCheckState = true; // (probably being called from the audio thread, so avoid blocking in here)
+    noPendingUpdates.store (false);
 }
 
 //==============================================================================
@@ -710,11 +704,11 @@ void MidiKeyboardComponent::updateNoteUnderMouse (const MouseEvent& e, bool isDo
 
 void MidiKeyboardComponent::updateNoteUnderMouse (Point<float> pos, bool isDown, int fingerNum)
 {
-    float mousePositionVelocity = 0.0f;
-    auto newNote = xyToNote (pos, mousePositionVelocity);
-    auto oldNote = mouseOverNotes.getUnchecked (fingerNum);
-    auto oldNoteDown = mouseDownNotes.getUnchecked (fingerNum);
-    auto eventVelocity = useMousePositionForVelocity ? mousePositionVelocity * velocity : velocity;
+    const auto noteInfo = xyToNote (pos);
+    const auto newNote = noteInfo.note;
+    const auto oldNote = mouseOverNotes.getUnchecked (fingerNum);
+    const auto oldNoteDown = mouseDownNotes.getUnchecked (fingerNum);
+    const auto eventVelocity = useMousePositionForVelocity ? noteInfo.velocity * velocity : velocity;
 
     if (oldNote != newNote)
     {
@@ -758,8 +752,7 @@ void MidiKeyboardComponent::mouseMove (const MouseEvent& e)
 
 void MidiKeyboardComponent::mouseDrag (const MouseEvent& e)
 {
-    float mousePositionVelocity;
-    auto newNote = xyToNote (e.position, mousePositionVelocity);
+    auto newNote = xyToNote (e.position).note;
 
     if (newNote >= 0 && mouseDraggedToKey (newNote, e))
         updateNoteUnderMouse (e, true);
@@ -771,8 +764,7 @@ void MidiKeyboardComponent::mouseUpOnKey      (int, const MouseEvent&)  {}
 
 void MidiKeyboardComponent::mouseDown (const MouseEvent& e)
 {
-    float mousePositionVelocity;
-    auto newNote = xyToNote (e.position, mousePositionVelocity);
+    auto newNote = xyToNote (e.position).note;
 
     if (newNote >= 0 && mouseDownOnKey (newNote, e))
         updateNoteUnderMouse (e, true);
@@ -782,8 +774,7 @@ void MidiKeyboardComponent::mouseUp (const MouseEvent& e)
 {
     updateNoteUnderMouse (e, false);
 
-    float mousePositionVelocity;
-    auto note = xyToNote (e.position, mousePositionVelocity);
+    auto note = xyToNote (e.position).note;
 
     if (note >= 0)
         mouseUpOnKey (note, e);
@@ -810,19 +801,17 @@ void MidiKeyboardComponent::mouseWheelMove (const MouseEvent&, const MouseWheelD
 
 void MidiKeyboardComponent::timerCallback()
 {
-    if (shouldCheckState)
+    if (noPendingUpdates.exchange (true))
+        return;
+
+    for (int i = rangeStart; i <= rangeEnd; ++i)
     {
-        shouldCheckState = false;
+        bool isOn = state.isNoteOnForChannels (midiInChannelMask, i);
 
-        for (int i = rangeStart; i <= rangeEnd; ++i)
+        if (keysCurrentlyDrawnDown[i] != isOn)
         {
-            bool isOn = state.isNoteOnForChannels (midiInChannelMask, i);
-
-            if (keysCurrentlyDrawnDown[i] != isOn)
-            {
-                keysCurrentlyDrawnDown.setBit (i, isOn);
-                repaintNote (i);
-            }
+            keysCurrentlyDrawnDown.setBit (i, isOn);
+            repaintNote (i);
         }
     }
 }
