@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -46,6 +45,26 @@ public:
     virtual void globalFocusChanged (Component* focusedComponent) = 0;
 };
 
+//==============================================================================
+/**
+    Classes can implement this interface and register themselves with the Desktop class
+    to receive callbacks when the operating system dark mode setting changes. The
+    Desktop::isDarkModeActive() method can then be used to query the current setting.
+
+    @see Desktop::addDarkModeSettingListener, Desktop::removeDarkModeSettingListener,
+         Desktop::isDarkModeActive
+
+    @tags{GUI}
+*/
+class JUCE_API  DarkModeSettingListener
+{
+public:
+    /** Destructor. */
+    virtual ~DarkModeSettingListener() = default;
+
+    /** Callback to indicate that the dark mode setting has changed. */
+    virtual void darkModeSettingChanged() = 0;
+};
 
 //==============================================================================
 /**
@@ -136,22 +155,50 @@ public:
     */
     void addGlobalMouseListener (MouseListener* listener);
 
-    /** Unregisters a MouseListener that was added with the addGlobalMouseListener()
-        method.
+    /** Unregisters a MouseListener that was added with addGlobalMouseListener().
 
         @see addGlobalMouseListener
     */
     void removeGlobalMouseListener (MouseListener* listener);
 
     //==============================================================================
-    /** Registers a MouseListener that will receive a callback whenever the focused
+    /** Registers a FocusChangeListener that will receive a callback whenever the focused
         component changes.
+
+        @see removeFocusChangeListener
     */
     void addFocusChangeListener (FocusChangeListener* listener);
 
-    /** Unregisters a listener that was added with addFocusChangeListener(). */
+    /** Unregisters a FocusChangeListener that was added with addFocusChangeListener().
+
+        @see addFocusChangeListener
+    */
     void removeFocusChangeListener (FocusChangeListener* listener);
 
+    //==============================================================================
+    /** Registers a DarkModeSettingListener that will receive a callback when the
+        operating system dark mode setting changes. To query whether dark mode is on
+        use the isDarkModeActive() method.
+
+        @see isDarkModeActive, removeDarkModeSettingListener
+    */
+    void addDarkModeSettingListener (DarkModeSettingListener* listener);
+
+    /** Unregisters a DarkModeSettingListener that was added with addDarkModeSettingListener().
+
+        @see addDarkModeSettingListener
+    */
+    void removeDarkModeSettingListener (DarkModeSettingListener* listener);
+
+    /** True if the operating system "dark mode" is active.
+
+        To receive a callback when this setting changes implement the DarkModeSettingListener
+        interface and use the addDarkModeSettingListener() to register a listener.
+
+        @see addDarkModeSettingListener, removeDarkModeSettingListener
+    */
+    bool isDarkModeActive() const;
+
     //==============================================================================
     /** Takes a component and makes it full-screen, removing the taskbar, dock, etc.
 
@@ -326,6 +373,10 @@ public:
     bool isOrientationEnabled (DisplayOrientation orientation) const noexcept;
 
     //==============================================================================
+    /** Returns the Displays object representing the connected displays.
+
+        @see Displays
+    */
     const Displays& getDisplays() const noexcept        { return *displays; }
 
     //==============================================================================
@@ -343,11 +394,16 @@ public:
     /** True if the OS supports semitransparent windows */
     static bool canUseSemiTransparentWindows() noexcept;
 
-   #if JUCE_MAC
-    /** OSX-specific function to check for the "dark" title-bar and menu mode. */
-    static bool isOSXDarkModeActive();
+   #if JUCE_MAC && ! defined (DOXYGEN)
+    [[deprecated ("This macOS-specific method has been deprecated in favour of the cross-platform "
+                  " isDarkModeActive() method.")]]
+    static bool isOSXDarkModeActive()  { return Desktop::getInstance().isDarkModeActive(); }
    #endif
 
+    //==============================================================================
+    /** Returns true on a headless system where there are no connected displays. */
+    bool isHeadless() const noexcept;
+
 private:
     //==============================================================================
     static Desktop* instance;
@@ -363,6 +419,7 @@ private:
 
     ListenerList<MouseListener> mouseListeners;
     ListenerList<FocusChangeListener> focusListeners;
+    ListenerList<DarkModeSettingListener> darkModeSettingListeners;
 
     Array<Component*> desktopComponents;
     Array<ComponentPeer*> peers;
@@ -410,6 +467,14 @@ private:
     Desktop();
     ~Desktop() override;
 
+    //==============================================================================
+    class NativeDarkModeChangeDetectorImpl;
+    std::unique_ptr<NativeDarkModeChangeDetectorImpl> nativeDarkModeChangeDetectorImpl;
+
+    static std::unique_ptr<NativeDarkModeChangeDetectorImpl> createNativeDarkModeChangeDetectorImpl();
+    void darkModeChanged();
+
+    //==============================================================================
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Desktop)
 };
 
