@@ -535,9 +535,10 @@ public:
         {
             // need to set a dummy represented file here to show the file icon (which we then set to the new icon)
             if (! windowRepresentsFile)
-                [window setRepresentedFilename:juceStringToNS (" ")]; // can't just use an empty string for some reason...
+                [window setRepresentedFilename: juceStringToNS (" ")]; // can't just use an empty string for some reason...
 
-            [[window standardWindowButton:NSWindowDocumentIconButton] setImage:imageToNSImage (ScaledImage (newIcon))];
+            auto img = NSUniquePtr<NSImage> { imageToNSImage (ScaledImage (newIcon)) };
+            [[window standardWindowButton: NSWindowDocumentIconButton] setImage: img.get()];
         }
     }
 
@@ -925,7 +926,7 @@ public:
                 }
 
                 detail::ColorSpacePtr colourSpace { CGColorSpaceCreateWithName (kCGColorSpaceSRGB) };
-                CGImageRef image = juce_createCoreGraphicsImage (temp, colourSpace.get(), false);
+                CGImageRef image = juce_createCoreGraphicsImage (temp, colourSpace.get());
                 CGContextConcatCTM (cg, CGAffineTransformMake (1, 0, 0, -1, r.origin.x, r.origin.y + clipH));
                 CGContextDrawImage (cg, CGRectMake (0.0f, 0.0f, clipW, clipH), image);
                 CGImageRelease (image);
